#ifndef DATABASEDELEGATE_H
#define DATABASEDELEGATE_H
#include <ostream>
#include <QStandardPaths>
#include <QFile>
#include <QDir>
#include <QDebug>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>
#include "utils/stringutils.h"
#include "pojo/jsonvalue.h"
#include "utils/jsonutils.h"
#include "utils/commonlog.h"

using namespace std;
class DataBaseDelegate{
    QString _dataBaseFilePath = "";
    QJsonArray _array;

    QString _dataBaseName;
    bool _readed = false;
public:
    DataBaseDelegate(){}

    bool createDataBaseIfNoExistStd(const std::string& baseName,const std::string& pPath=""){
        return createDataBaseIfNoExist(QString(baseName.c_str()),QString(pPath.c_str()));
    }

    bool createDataBaseIfNoExist(const QString& baseName,const QString& pPath=""){
        this->_dataBaseName = baseName;
        QString basePath = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
        QString dataBasePath = basePath + pPath;

        QDir folder;
        if(!folder.exists(dataBasePath)){
            bool suc = folder.mkpath(dataBasePath);
            if(!suc)return false;
        }

        _dataBaseFilePath = dataBasePath + "/" + baseName;
//        printf("dataBasePath %s\n",_dataBaseFilePath.toStdString().c_str());
        return true;
    }

    bool useDataBaseStd(const string & databaseName,bool read = false,const string& pPath=""){
        return useDataBase(QString(databaseName.c_str()),read,QString(pPath.c_str()));
    }

    bool useDataBase(const QString & databaseName,bool read = false,const QString& pPath=""){
        destroy();
        bool suc = createDataBaseIfNoExist(databaseName,pPath);

        if(!suc)return suc;
//        if(read){
//            openAndReadAll();
//        }
        return true;
    }

    bool openAndReadAllStd(const std::function<string (char*)>& decode){
        return openAndReadAll(decode);
    }

    bool openAndReadAll(const std::function<string (char*)>& decode =[](char* d)->string{return d;}){
        if(_readed)return false;
        _readed = true;
        QFile file (_dataBaseFilePath);
        if(!file.open(QIODevice::ReadOnly)){
            return false;
        }
        QByteArray datas = file.readAll();
        if(!datas.isNull() && !datas.isEmpty()){
            if(decode)datas = QByteArray(decode(datas.data()).c_str());
            QJsonParseError err_rpt;
            QJsonDocument root_Doc = QJsonDocument::fromJson(datas, &err_rpt);

            if(err_rpt.error != QJsonParseError::NoError)
            {
              QFile file(_dataBaseFilePath);
                if(!file.remove())
                    return false;
                return useDataBase(_dataBaseName);
            }
            _array = root_Doc.array();
        }
        return true;
    }

    void destroy(){

     }

    void addDatasNoSaveStd(const map<string,JsonValue>& map){
        QVariantMap qmap;
        auto iter = map.begin();
        while(iter != map.end()){
            qmap[QString(iter->first.c_str())] = JsonUtils::jsonValue2variant(iter->second);
            iter ++;
        }
        addDatasNoSave(qmap);
    }

    void addDatasNoSave(const QVariantMap& map){
        openAndReadAll();
        QJsonDocument doc = QJsonDocument::fromVariant(map);
        _array.append(doc.object());
    }

    bool reSaveStd(const vector<map<string,JsonValue>>& datas,const std::function<string (char*)>& encodeFunc){
        QVariantList list;
        for(map<string,JsonValue> s : datas){
            QMap<QString,QVariant> qmap;
            auto iter = s.begin();
            while(iter != s.end()){
                qmap[QString(iter->first.c_str())] = JsonUtils::jsonValue2variant(iter->second);
                iter++;
            }

            list.push_back(QVariant(qmap));
        }

        return reSave(list,encodeFunc);
    }

    bool reSave(const QVariantList& datas,const std::function<string (char*)>& encodeFunc = [](char* d)->string{return d;}){
        if(datas.isEmpty())return false;

        QJsonDocument jsonDoc;
        _array = QJsonArray::fromVariantList(datas);
        return save(encodeFunc);
    }

    bool reSaveOneDataStd(const map<string,JsonValue>& data,const std::function<string (char*)>& encodeFunc){
        QVariantMap qmap;
        auto iter = data.begin();
        while(iter != data.end()){
            qmap[QString(iter->first.c_str())] = JsonUtils::jsonValue2variant(iter->second);
            iter ++;
        }

        return reSaveOneData(qmap,encodeFunc);
    }

    bool reSaveOneData(const QVariantMap& data,const std::function<string (char*)>& encodeFunc = [](char* d)->string{return string(d);}){
        if(data.isEmpty())return false;
//        qDebug () << "reSaveOneData";
        QJsonDocument jsonDoc;
        QVariantList list;
        list.append(data);
        _array = QJsonArray::fromVariantList(list);
        return save(encodeFunc);
    }

    bool save(const std::function<string (char*)>& encodeFunc = [](char* d)->string{return string(d);}){
//        qDebug () << "save";
        if(!_array.isEmpty()){
            QJsonDocument jsonDoc;
            jsonDoc.setArray(_array);
            QFile file(_dataBaseFilePath);
            if(file.open(QIODevice::WriteOnly | QIODevice::Text)){
                QByteArray arr = jsonDoc.toJson();
//                qDebug() << "保存" <<QString(arr);
                string endata = encodeFunc(arr.data());
                auto len = file.write(endata.c_str());
//                qDebug() << "保存加密" << endata.c_str();
                KT_Debug("write len %d ",len);
                if(len == 0){
                    KT_Error("保存数据 write len == 0");
                }
                file.close();
                return true;
            }
         }
        destroy();
        return false;
    }

    QVariantList getAll(const std::function<string (char*)>& decode=[](char* d)->string{return d;}){
        openAndReadAll(decode);
      return _array.toVariantList();
    }

    map<string,JsonValue> getFirstStd(const std::function<string (char*)>& decode){
        QVariantMap qmap = getFirst(decode);
        map<string,JsonValue> map;
        auto iter = qmap.begin();
        while(iter != qmap.end()){
            map[iter.key().toStdString()] = JsonUtils::variant2JsonValue(iter.value());
            iter++;
        }
        return map;
    }

    QVariantMap getFirst(const std::function<string (char*)>& decode=[](char* d)->string{return d;}){
        openAndReadAll(decode);
        if(_array.isEmpty())return QVariantMap();
        QVariant v = _array.first();
        return v.value<QVariantMap>();
    }

};

#endif
