#include "aboutwindow.h"
#include <QScreen>
#include <QGuiApplication>
#include "loginwindow.h"
#include <QVBoxLayout>
#include <QLabel>
#include <QBitmap>
#include <QPainter>
#include <QPainterPath>
#include <QDateTime>
#include <QGraphicsDropShadowEffect>
#include <QHBoxLayout>
#include <QPalette>

#include "dataManager/cappsetting.h"
#include "QPalette"
#include "utils/AppUtils.h"
#include "QPoint"
#include "QPainterPath"
#include "utils/jsonutils.h"
#include "presenters/cproductpresenter.h"
#include "selectcountrycodewindow.h"
#include "mainwindow.h"
#include "accountselectwindow.h"
#include "widgets/clickedlabel.h"
#include "QPushButton"

using namespace std;
extern CAppSetting* g_cappSetting;

void AboutWindow::setMainWindow(MainWindow* host){
    this->mHost = host;
}

AboutWindow::AboutWindow(QWidget * container,QSize& size,int windowBlurRadius):QWidget(container)
{
    setFixedSize(size);
    this->_windowBlurRadius = windowBlurRadius;

    this->_container = this;
    _labelFont.setPixelSize(15);
    _labelFont.setWeight(0);

    QVBoxLayout *vBox = new QVBoxLayout;

    vBox->setContentsMargins(0,0,0,0);

    vBox->setSpacing(0);
    QWidget* coverImage = addCoverImage(_container,windowBlurRadius);//顶部巨幅
    vBox->addWidget(coverImage);
    vBox->setContentsMargins(0,0,0,0);
    setLayout(vBox);

    QVBoxLayout* bottomLay = new QVBoxLayout;
    bottomLay->setAlignment(Qt::AlignCenter);

    QVBoxLayout* contentLay = new QVBoxLayout;
    contentLay->setContentsMargins(35,19,0,0);
    contentLay->setSpacing(3);

    QLabel* vlabel = new QLabel;
    vlabel->setText("版本号："+QString(CAppSetting::getVersionName().c_str()));
    vlabel->setStyleSheet("QLabel{color:#444444}");
    vlabel->setFont(_labelFont);
    contentLay->addWidget(vlabel);

    QLabel* comlabel = new QLabel;
    comlabel->setFont(_labelFont);
    comlabel->setStyleSheet("QLabel{color:#444444}");
    comlabel->setText("版权所有：(C)"+QString::number(QDateTime::currentDateTime().date().year())+"金蝶蝶金云计算有限公司");
    contentLay->addWidget(comlabel);

    QHBoxLayout* devLay = new QHBoxLayout;
    devLay->setContentsMargins(0,0,0,0);
    devLay->setSpacing(0);

    QLabel* devNameLabel = new QLabel;
    devNameLabel->setText("设备号：");
    devNameLabel->setStyleSheet("QLabel{color:#444444}");
    devNameLabel->setFont(_labelFont);
    devLay->addWidget(devNameLabel);

    QLineEdit* devNameLine = new QLineEdit;
    devNameLine->setFont(_labelFont);
    devNameLine->setStyleSheet("QLineEdit{border:none;color:#444444}");
    devNameLine->setReadOnly(true);
    devNameLine->setText(QString(g_cappSetting->getDeviceId().c_str()));
    devLay->addWidget(devNameLine);

    contentLay->addLayout(devLay);

    QHBoxLayout* optLay = new QHBoxLayout;
    optLay->setContentsMargins(10,14,45,0);
    QPushButton* kisyunBtn = new QPushButton;
    kisyunBtn->setFont(_labelFont);
    kisyunBtn->setFixedHeight(30);
    kisyunBtn->setText("金蝶KIS云官网");
    connect(kisyunBtn,&QPushButton::clicked,this,[=]{
        AppUtils::openUrlByDefaultBrower(QString("https://kisyun.kingdee.com"));
    });

    kisyunBtn->setStyleSheet("QPushButton{border:none;border-radius:4px;padding:5px 6px;background-color:#cc74caf7;color:#ffffff}"
                             "QPushButton:hover{background-color:rgb(116,202,247);}");
    QPushButton* userxBtn = new QPushButton;
    userxBtn->setText("查看用户协议");
    userxBtn->setFont(_labelFont);
    userxBtn->setStyleSheet("QPushButton{border:none;border-radius:4px;padding:5px 6px;background-color:#cc74caf7;color:#ffffff}"
                                 "QPushButton:hover{background-color:rgb(116,202,247);}");
    connect(userxBtn,&QPushButton::clicked,this,[=]{
        AppUtils::openUrlByDefaultBrower(QString("https://cloud.kingdee.com/passport/#/policy-agree/client-kd"));
    });
    userxBtn->setFixedHeight(30);

    optLay->addWidget(kisyunBtn,1);
    optLay->addSpacing(10);
    optLay->addWidget(userxBtn,1);
    contentLay->addLayout(optLay);

    bottomLay->addLayout(contentLay);

    vBox->addLayout(bottomLay);

//    vBox->addStretch();
    QWidget* pageWidget = new QWidget(_container);//窗口content
    pageWidget->setLayout(vBox);

    ClickedLabel* backBtn = new ClickedLabel(pageWidget);
    backBtn->setText("关闭");
    backBtn->setStyleSheet("QLabel{color:#ffffff}");

    backBtn->setGeometry(size.width() - 55,0,55,47);
    connect(backBtn,&ClickedLabel::clicked,this,[=]{
        mHost->setShowMode(mLastMainMode);
    });
}

QLabel * AboutWindow::addCoverImage(QWidget * container,int windowBlurRadius){
    QLabel * converImage = new QLabel;
    QPixmap pixmap(":/images/login_banner.png");

    double pr = QGuiApplication::primaryScreen()->devicePixelRatio();

    QPixmap sp = pixmap.scaled(container->width() * pr,1,Qt::KeepAspectRatioByExpanding);
    sp.setDevicePixelRatio(pr);

    int off = sp.height() * .07;
    QPixmap map(sp.width(),sp.height() - off * 2);

    map.fill(Qt::transparent);
    map.setDevicePixelRatio(sp.devicePixelRatio());
    QPainter mapPainter(&map);
    QPainterPath path;
    mapPainter.setPen(Qt::NoPen);
    path.addRoundedRect(0,0,container->width(),sp.height() ,windowBlurRadius,windowBlurRadius);
    mapPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    mapPainter.setClipPath(path);
    mapPainter.drawPixmap(0,-off/2,sp);
    mapPainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);

    QPixmap conver(sp.width(),sp.height());
    conver.setDevicePixelRatio(pr);
    conver.fill(Qt::transparent);
    QPainter converPainter(&conver);
    converPainter.setPen(Qt::NoPen);
    converPainter.setBrush(Qt::black);
    converPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    converPainter.drawRoundedRect(0,0,container->width(),sp.height(),windowBlurRadius,windowBlurRadius);
    mapPainter.drawPixmap(0,0,conver);

    converImage->setPixmap(map);
    return converImage;
}



