#include "accountselectwindow.h"
#include <QtSvg>
#include <QVBoxLayout>
#include <QLabel>
#include <QBitmap>
#include <QPainter>
#include <QApplication>
#include <QScreen>
#include <QPainter>
#include "widgets/windowclosebtn.cpp"
#include <QGraphicsDropShadowEffect>
#include "widgets/windowminbtn.cpp"
#include <QHBoxLayout>
#include <QPalette>
#include <QSvgRenderer>
#include "widgets/QMCheckBox.cpp"
#include "widgets/QMLineEdit.h"
#include "dataManager/cappsetting.h"
#include "QPalette"
#include "utils/AppUtils.h"
#include "QPoint"
#include "loginprivwindow.h"

#include "utils/jsonutils.h"

#include "selectcountrycodewindow.h"
#include "mainwindow.h"

#include "presenters/accountpresenter.h"
#include "pojo/enterpriseaccinfo.h"
#include "widgets/adjustlengthpushbutton.h"
#include "utils/mainthreadhandler.h"
#include "widgets/expandlistpop.h"
#include "utils/stringutils.h"
#include "manager/rdpmanager.h"
#include "dataManager/cloginhistories.h"

using namespace std;
extern CAppSetting* g_cappSetting;

void AccountSelectWindow::setMainWindow(MainWindow* host){
    this->mHost = host;
}

AccountSelectWindow::~AccountSelectWindow(){
    if(mEnterpriseInfo.size() >0){
       for(auto& item : mEnterpriseInfo){
           delete item;
       }
   }
}

void AccountSelectWindow::setRadius(int radius){
    auto child = mCoverContainer->findChild<QWidget*>("coverImage");

    QWidget* cover = addCoverImage(_container,radius);

    mCoverContainer->layout()->removeWidget(child);
    mCoverContainer->layout()->addWidget(cover);
    child->setParent(nullptr);
    child->deleteLater();
    cover->setObjectName("coverImage");
}

AccountSelectWindow::AccountSelectWindow(QWidget * container,QSize size,int windowBlurRadius):QWidget(container)
{
    setFixedSize(size);
    this->_windowBlurRadius = windowBlurRadius;
    this->_container = this;
    _labelFont.setPixelSize(15);
    _labelFont.setWeight(0);

//    _labelFont.setFamily("宋体");

    QVBoxLayout *vBox = new QVBoxLayout;

    vBox->setContentsMargins(0,0,0,0);

    vBox->setSpacing(0);
    QWidget* coverImage = addCoverImage(_container,windowBlurRadius);//顶部巨幅
    coverImage->setObjectName("coverImage");
    mCoverContainer = new QWidget();
    QHBoxLayout* containerLay = new QHBoxLayout(mCoverContainer);
    containerLay->setContentsMargins(0,0,0,0);
    containerLay->setSpacing(0);
    containerLay->addWidget(coverImage);
    vBox->addWidget(mCoverContainer);

    vBox->setContentsMargins(0,0,0,0);
    vBox->setSpacing(0);
    vBox->addSpacing(16);
    vBox->addWidget(enterprSelectItems(_container));
    vBox->addWidget(_enterpriseLine = line(_container));
    vBox->addSpacing(12);
    vBox->addWidget(accSelectItems(_container));
    vBox->addWidget(_accountLine = line(_container));
    vBox->addSpacing(17);

    vBox->addWidget(loginBtn(_container));
    setLayout(vBox);

    vBox->addStretch();
    QWidget* pageWidget = new QWidget(_container);//窗口content
    pageWidget->setLayout(vBox);

    ImageView* qrBtn = new ImageView(pageWidget);
    connect(qrBtn,&ImageView::clicked,this,[=]{
        if(mAlertDialog&&mAlertDialog->isVisible())return;
        mHost->setShowMode(3);
    });

    _loginType = new QLabel(_container);
    QFont font;
    font.setPixelSize(14);
    _loginType->setFont(font);
    QPalette palette;
    palette.setColor(QPalette::WindowText,Qt::white);
    _loginType->setPalette(palette);
    changeLoginTypeLabel();
    _loginType->setGeometry(10,5,160,40);
    changeLoginTypeLabel();

    _toast = new ToastWidget(this);
}

QString AccountSelectWindow::getAccLabelText(AccountData* accData){
    if(!accData)return "";
    return QString(accData->acctname.c_str())+"-"+QString(accData->productname.c_str());
}

void AccountSelectWindow::setData(QString acc,QString paw,vector<EnterpriseAccInfo *> info,const QString uid,const QString token){
    mEnterpriseInfo.clear();
    changeLoginTypeLabel();
    mEnterpriseInfo.insert(mEnterpriseInfo.end(),info.begin(),info.end());
    mEnterpriseInfo = info;
    this->uid = uid;
    mAcc = acc;
    mPaw = paw;
    string stdUid = uid.toStdString();
    CLoginHistories history;
    vector<string> selectedTidAccId = history.getLoginAccounts(stdUid);

    this->token = token;
    if(mEnterpriseInfo.size() > 0){
        int tSelectIndex = -1;
        int acSelectIndex = -1;

        if(selectedTidAccId.size() == 2){
            int index = 0;
            string selectTid = selectedTidAccId[0];
            string selectAccId = selectedTidAccId[1];
            for(EnterpriseAccInfo* accInfo : info){
                if(accInfo->tid == selectTid){
                    tSelectIndex = index;
                    int aIndex = 0;
                    for(AccountData* acc : accInfo->acctdata){
                        if(acc->acctid == selectAccId){
                            acSelectIndex = aIndex;
                            break;
                        }
                        aIndex ++;
                    }
                    break;
                }
                index ++;
            }
        }

        if(tSelectIndex == -1 || acSelectIndex == -1){
            tSelectIndex = 0;
            acSelectIndex = 0;
        }

        mSelectedAccIndex = acSelectIndex;
        mSelectedEnterpriseIndex = tSelectIndex;
        EnterpriseAccInfo * item = info.at(tSelectIndex);
        mEnterpriseLabel->setText(QString(item->name.c_str()));
        auto fData = item->acctdata.at(acSelectIndex);
        mAccLabel->setText(getAccLabelText(fData));
    }
}

QSize AccountSelectWindow::windowSize(){
    return QSize(420,313);
}

void AccountSelectWindow::changeLoginTypeLabel(){
    _loginType->setText(g_cappSetting->isExpMode() ? "登录-公有云-体验版" : "登录-公有云");
}

//void AccountSelectWindow::onItemClick(ExpandListPop * p , QString& text,int index) {
//    if(p == mEnterPrisePop){
//
//    }else if(p == mAccPop){
//
//    }
//    p->dismiss();
//}

QSize AccountSelectWindow::getTargetSize(){
    return QSize(431,331);
}

QWidget* AccountSelectWindow::line (QWidget * container){
    ImageView* split = new ImageView;
    split->setFixedSize(container->width() - _lrpadding * 2,1);
    split->setBackgroudColor(_borderColor);
    QHBoxLayout* lay = new QHBoxLayout;
    lay->setContentsMargins(0,0,0,0);
    lay->setSpacing(0);

    lay->addWidget(split);
    QWidget* line = new QWidget;
    line->setLayout(lay);
    return line;
}

void AccountSelectWindow::closeEvent(QCloseEvent *event){
    if(!mConnectSuc && mRdpManagerPtr.get()){
//        printf("直接释放链接 \n");
        int rdpCode = mRdpManagerPtr->getRdpCode();
        if(rdpCode > -1){
//            printf("直接释放链接%d \n",rdpCode);
            RemoteDesktopManager::getInstance()->destroyRdpCode(rdpCode);
        }
        mRdpManagerPtr = nullptr;
    }
}

QWidget * AccountSelectWindow::accSelectItems(QWidget * container){
    QWidget* content = new QWidget;
    int itmHeight = 25;
    QHBoxLayout* layout = new QHBoxLayout;
    layout->setContentsMargins(_lrpadding,0,_lrpadding,0);

    layout->setSizeConstraint(QLayout::SetMaximumSize);
    layout->setSpacing(0);

    mAccLabel = new AdjustLengthPushButton();
    mAccLabel->setFont(_labelFont);
    mAccLabel->setStyleSheet("color:#555555;");

    mAccLabel->setFixedWidth(container->width() - _lrpadding * 2 - 35);

    mAccLabel->setFixedHeight(itmHeight);

    connect(mAccLabel,&ClickedLabel::clicked,this,[=]{
        showAccInfos();
    });
//    mAccLabel->setAttribute(Qt::WA_MacShowFocusRect,false);
//    mAccLabel->setStyleSheet("QPushButton{text-align:left;border:none;color:#666666}"
//                              "QPushButton:pressed{text-align:left;background-color: rgb(255, 255, 255);border:none;}");
//    mAccLabel->setFlat(true);
    layout->addWidget(mAccLabel);

    ImageView * accExpBtn = comIconBtn(":/images/expand_more.svg");
    layout->addWidget(accExpBtn);

    connect(accExpBtn,&ImageView::clicked,this,[=]{
        showAccInfos();
    });

    content->setLayout(layout);
    return content;
}

QWidget * AccountSelectWindow::loginBtn(QWidget * container){
    QHBoxLayout* lay = new QHBoxLayout;
    ImageView * btn = new ImageView;
    btn->setFixedSize(container->width() - _lrpadding * 2,29);
    btn->setBackgroudColor(QColor(91,201,251,150));
    btn->setLabel(QObject::tr("login"),"login");
    btn->setClipRadius(5);
    btn->setFontColor(Qt::white);
    btn->setFont(_labelFont);
    lay->addWidget(btn);
    lay->setSpacing(0);
    lay->setContentsMargins(0,5,0,0);
    QWidget *w = new QWidget;
    w->setLayout(lay);

    btn->setOnClickListener(this);
    return w;
}

void AccountSelectWindow::onClick(ImageView * ,QMouseEvent*){
    if(mAlertDialog&&mAlertDialog->isVisible())return;
//    ProductPresenter * prsenter = new ProductPresenter(this);

    EnterpriseAccInfo* eInfo = mEnterpriseInfo[mSelectedEnterpriseIndex];
    AccountData* accData = eInfo->acctdata[mSelectedAccIndex];
    _progdialog = new ProgressDialog(this);
    _progdialog->setRadius(g_cappSetting->getWindowRadius());
     _progdialog->showDialog("进入账套中..");
    mRdpManagerPtr = make_unique<RdpManager>();
    mRdpManagerPtr.get()->enterRdp(
                mAcc.toStdString(),mPaw.toStdString(),
                uid.toStdString(),token.toStdString(),
                eInfo->tid,
                accData,
                [=](ReponseResult<RdpConnectResult>& connectResult){
        if(!connectResult.isSuc()){
              _progdialog->dismiss();
              _toast->showMesssage(QString(connectResult.getErrMsg().c_str()));
          }else{
            RdpConnectResult connRes = connectResult.getData();
            if(!connRes.isFinished){
                _progdialog->setLabel(QString(connRes.msg.c_str()));
            }else{
                _progdialog->dismiss();
                if(connRes.isWaring){
                    AlertDialog* alert = new AlertDialog(this);

                    alert->setFixedSize(this->size());
                    alert->setRadius(_windowBlurRadius);
                    vector<QString> btns ;
                    if(connRes.canVisAcc){
                        btns = {"取消","确定"};
                    }else{
                        btns = {"确定"};
                    }
                    alert->setButtons(btns);
                    alert->setTitle("提示");
                    alert->setMessage(QString(connRes.msg.c_str()));
                    connect(alert,&AlertDialog::buttonClick,this,[=](int index){
                        alert->dismiss();
                        alert->setParent(nullptr);
                        alert->deleteLater();
                        if(index == 1){
                            _progdialog = new ProgressDialog(this);
                            _progdialog->setRadius(g_cappSetting->getWindowRadius());
                             _progdialog->showDialog("进入账套中..");
                             mRdpManagerPtr.get()->enterAcc();
                        }
                    });

                    alert->show();
                }else{
                    _progdialog->dismiss();
                    if(connRes.success){
                        CLoginHistories history;
                        EnterpriseAccInfo* eInfo = mEnterpriseInfo[mSelectedEnterpriseIndex];
                        AccountData* accData = eInfo->acctdata[mSelectedAccIndex];
                        string uidStd = this->uid.toStdString();
                        string tidStd = eInfo->tid;
                        string acidStd = accData->acctid;
                        mConnectSuc = true;
                        history.addLoginAccount(uidStd,tidStd,acidStd);
                        mHost->closeMainWindow();
                    }else{
                        _toast->showMesssage(QString(connRes.msg.c_str()));
                    }
                }
            }
        }
    }
 );
}

QWidget* AccountSelectWindow::enterprSelectItems(QWidget * container){
    QWidget* content = new QWidget;
    int itmHeight = 25;

    QHBoxLayout* layout = new QHBoxLayout;
    layout->setContentsMargins(_lrpadding,0,_lrpadding,0);

    layout->setSizeConstraint(QLayout::SetMaximumSize);
    layout->setSpacing(0);

//    mCountryCode = new ImageView;
//    connect(mCountryCode,&ImageView::clicked,this,[=]{
//        SelectCountryCodeWindow* window = new SelectCountryCodeWindow;
//        connect(window,&SelectCountryCodeWindow::selected,this,[=](SelectCountryCodeWindow::CountryCode code){
//            mCountryCode->setLabel("+"+QString::number(code.num),"");
//        });

//        QPoint point = mHost->pos();
//        int x = point.x();
//        int y = point.y();
//        int showX ;
//        if(x < window->width()*1.5){
//            showX = x + mHost->width() - _windowBlurRadius+1;
//        }else{
//            showX = x - window->width()+_windowBlurRadius-1;
//        }
//        window->show(showX,y + _windowBlurRadius);
//    });

//    mCountryCode->setLabel("+86","");
//    mCountryCode->waveCenter(true);
//    mCountryCode->setRadius(100);
//    mCountryCode->setLabelFont(_labelFont);
//    mCountryCode->setFontColor(_labelColor);
//    mCountryCode->setFixedSize(41,41);

//    layout->addWidget(mCountryCode);
//    layout->addSpacing(5);
    ImageView * split = new ImageView;
    split->setFixedSize(1,itmHeight);
    split->setBackgroudColor(_borderColor);

//    layout->addWidget(split);
//    layout->addSpacing(10);

    mEnterpriseLabel = new AdjustLengthPushButton;
    mEnterpriseLabel->setStyleSheet("color:#555555;");
    mEnterpriseLabel->setFont(_labelFont);
//    mEnterpriseLabel->setText("金蝶KIS云体验企业");
    QPalette palette;
    palette.setColor(QPalette::Text, _labelColor);
    mEnterpriseLabel->setPalette(palette);
    mEnterpriseLabel->setFixedHeight(itmHeight);

    mEnterpriseLabel->setAttribute(Qt::WA_MacShowFocusRect,false);

    layout->addWidget(mEnterpriseLabel);

    connect(mEnterpriseLabel,&ClickedLabel::clicked,this,[=]{
        showEnterPInfos();
    });

//     layout->addWidget(comIconBtn(":/images/clear.svg"));
    ImageView * accExpBtn = comIconBtn(":/images/expand_more.svg");
    accExpBtn->setTag(this);
    connect(accExpBtn,&ImageView::clicked,this,[=]{
        showEnterPInfos();
    });
    layout->addWidget(accExpBtn);
    content->setLayout(layout);
//    content->setFixedSize(container->width() , itmHeight);
    return content;
}

void AccountSelectWindow::showAccInfos(){
    if(mAlertDialog&&mAlertDialog->isVisible())return;
   QWidget * line = _accountLine;
    int l = _accountLine->pos().x();
    int t = _accountLine->pos().y();
    QWidget * parent = (QWidget*)line->parent();
    while(parent != nullptr && parent->isWidgetType()){
        l += parent->pos().x();
        t += parent->pos().y();
        parent = (QWidget*)parent->parent();
    }

    mAccPop = new ExpandListPop(323+30);
    QList<QString> list ;

    vector<AccountData*> datas = mEnterpriseInfo.at(mSelectedEnterpriseIndex)->acctdata;
    if(datas.size() == 0)return;
    for(auto& info : datas){
        list.push_back(getAccLabelText(info));
    }

    mAccPop->setItemPadding(15,0,15,0);
    mAccPop->setItems(list);
    mAccPop->showPop(l+_lrpadding-15,t);
    mAccPop->setOnItemClickCallback([&](const QString& label, int index){
        mAccLabel->setText(label);
        mSelectedAccIndex = index;
        mAccPop->dismiss();
    });
}

void AccountSelectWindow::showEnterPInfos(){
    if(mAlertDialog&&mAlertDialog->isVisible())return;
   QWidget * line = _enterpriseLine;
    int l = _enterpriseLine->pos().x();
    int t = _enterpriseLine->pos().y();
    QWidget * parent = (QWidget*)line->parent();
    while(parent != nullptr && parent->isWidgetType()){
        l += parent->pos().x();
        t += parent->pos().y();
        parent = (QWidget*)parent->parent();
    }

    mEnterPrisePop = new ExpandListPop(323+ 30);
    QList<QString> list ;
    for(auto& info : mEnterpriseInfo){
        list.push_back(QString(info->name.c_str()));
    }
    mEnterPrisePop->setItemPadding(15,0,15,0);
    mEnterPrisePop->setItems(list);
    mEnterPrisePop->showPop(l+_lrpadding-15,t);
    mEnterPrisePop->setOnItemClickCallback([&](const QString& label, int index){
        mEnterpriseLabel->setText(label);
       vector<AccountData*> accs = mEnterpriseInfo.at(index)->acctdata;
       mSelectedEnterpriseIndex = index;
       mSelectedAccIndex = 0;
       if(accs.size() > 0){
           mAccLabel->setText(getAccLabelText(accs[mSelectedAccIndex]));
       }
       mEnterPrisePop->dismiss();
    });
}

ImageView * AccountSelectWindow::comIconBtn(QString path){
    ImageView*imageView = new ImageView;
    imageView->setFixedSize(35,35);
    imageView->setRadius(100);
    imageView->waveCenter(true);
    imageView->setPadding(7,7,7,7);
    imageView->setSvgPixMapSrc(path,35,35);

    return imageView;
}

QLabel * AccountSelectWindow::addCoverImage(QWidget * container,int windowBlurRadius){
    QLabel * converImage = new QLabel;
    QPixmap pixmap(":/images/login_banner.png");

    double pr = QGuiApplication::primaryScreen()->devicePixelRatio();

    QPixmap sp = pixmap.scaled(container->width() * pr,1,Qt::KeepAspectRatioByExpanding);
    sp.setDevicePixelRatio(pr);

    int off = sp.height() * .07;
    QPixmap map(sp.width(),sp.height() - off * 2);
    mCoverSize = map.size();
    map.fill(Qt::transparent);
    map.setDevicePixelRatio(sp.devicePixelRatio());
    QPainter mapPainter(&map);
    QPainterPath path;
    mapPainter.setPen(Qt::NoPen);

    path.addRoundedRect(0,0,container->width(),sp.height() ,windowBlurRadius,windowBlurRadius);
    mapPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    mapPainter.setClipPath(path);
    mapPainter.drawPixmap(0,-off/2,sp);
    mapPainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);

    QPixmap conver(sp.width(),sp.height());
    conver.setDevicePixelRatio(pr);
    conver.fill(Qt::transparent);
    QPainter converPainter(&conver);
    converPainter.setPen(Qt::NoPen);
    converPainter.setBrush(Qt::black);
    converPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    converPainter.drawRoundedRect(0,0,container->width(),sp.height(),windowBlurRadius,windowBlurRadius);
    mapPainter.drawPixmap(0,0,conver);

    converImage->setPixmap(map);
    return converImage;
}


