#include "updatedialog.h"
#include <QPainter>
#include <QDesktopWidget>
#include <QApplication>
#include <QLabel>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QScrollBar>
#include <QPushButton>
#include <QSizePolicy>
#include <utils/AppUtils.h>

UpdateDialog::UpdateDialog(QWidget *parent) : QWidget(parent)
{
    setWindowFlags(Qt::FramelessWindowHint|Qt::ToolTip);
    setAttribute(Qt::WA_TranslucentBackground);
    setFixedSize(360,245);

    QDesktopWidget *desktop = QApplication::desktop();
    QRect rect = desktop->availableGeometry();
    move(rect.width() - width()-6,rect.height()-height()-6);

    QVBoxLayout* contentLay = new QVBoxLayout(this);

    int l = 25;
    int r = 17;
    contentLay->setContentsMargins(l,12,r,20);
    contentLay->setSpacing(0);

    mTitle = new QLabel();
    mTitle->setText("金蝶KIS云 更新提示");
    mTitle->setStyleSheet("QLabel{color:white;font-size:17px;}");
    contentLay->addWidget(mTitle);
    contentLay->addSpacing(5);

    QScrollArea* area = new QScrollArea;
    area->setFixedWidth(width()-l - r);
    area->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    area->setStyleSheet("QScrollArea{background-color:#00000000}");
    mContent = new QLabel();

    mContent->setFixedWidth(area->width()-5);

    mContent->setWordWrap(true);

//    mContent->setText("最新版本：1.0.1\n更新内容： \n1、修复了一些bug \n2、优化了用户体验");
    mContent->setStyleSheet("QLabel{background-color:#00000000;font-size:15px;color:#FFFFFF}");

    area->setWidget(mContent);

    QScrollBar* scrollBar = new QScrollBar(area);
    scrollBar->setStyleSheet("QScrollBar:vertical"
    "{"
        "width:4px;"
        "background:rgba(0,0,0,0%);"
        "margin:0px,0px,0px,0px;"
        "padding-top:0px;"   // 留出9px给上面和下面的箭头
        "padding-bottom:0px;"
    "}"
    "QScrollBar::handle:vertical"
    "{"
        "width:4px;"
        "background:rgba(0,0,0,25%);"
        "border-radius:2px;"   // 滚动条两端变成椭圆
        "min-height:20;"
    "}"
     "QScrollBar::add-line:vertical"   // 这个应该是设置下箭头的，3.png就是箭头
     "{"
         "height:9px;width:8px;"
         "border-image:url(:/images/a/3.png);"
         "subcontrol-position:bottom;"
     "}"
     "QScrollBar::sub-line:vertical"   // 设置上箭头
     "{"
         "height:9px;width:8px;"
         "border-image:url(:/images/a/1.png);"
         "subcontrol-position:top;"
     "}"
    );
    area->setVerticalScrollBar(scrollBar);
    contentLay->addWidget(area);
    contentLay->addSpacing(14);
    QHBoxLayout *lay = new QHBoxLayout(this);
    lay->setContentsMargins(10,0,10,0);
    mCloseBtn = getBtn("关闭");
    lay->addWidget(mCloseBtn,1);

    connect(mCloseBtn,&QPushButton::clicked,this,[=]{
        if(_isFocusUpdate)return;
        close();
        deleteLater();
    });
    lay->addSpacing(30);
    QPushButton * upBtn = getBtn("更新");
    lay->addWidget(upBtn,1);

    connect(upBtn,&QPushButton::clicked,this,[=]{
        AppUtils::openUrlByDefaultBrower(mDownUrl);
        if(!_isFocusUpdate){
            close();
            deleteLater();
        }
    });

    QWidget *w = new QWidget;
    w->setLayout(lay);

    contentLay->addWidget(w);
}

QPushButton* UpdateDialog::getBtn(const string & text){
    QPushButton* button = new QPushButton;
    button->setText(QString(text.c_str()));
    button->setStyleSheet("QPushButton{border:none;border-radius:4px;padding:5px 6px;background-color:#cc74caf7;color:#ffffff}"
                             "QPushButton:hover{background-color:rgb(116,202,247);}");
    return button;
}

void UpdateDialog::setTitle(const string& title){
    mTitle->setText(QString(title.c_str()));
}

void UpdateDialog::setMessage(string& msg){
    mContent->setText(QString(msg.c_str()));
    mContent->adjustSize();
}

void UpdateDialog::paintEvent(QPaintEvent *event) {
     QPainter painter(this);
     painter.setRenderHint(QPainter::Antialiasing);
     painter.setBrush(QColor(66,66,66,0xee));
     painter.setPen(Qt::NoPen);
     painter.drawRoundedRect(rect(),_radius,_radius);
}
