#include "fileutils.h"
#include "7zclient.h"
#include "QDir"
#include <QStandardPaths>
#include "utils/commonlog.h"
FileUtils::FileUtils()
{

}

string FileUtils::getHomePath(){
    return QStandardPaths::writableLocation(QStandardPaths::HomeLocation).toStdString();
}

int FileUtils::unCompress7zFile(const QString& inPath,const QString& outPath){
    QDir dir(outPath);
    if(!dir.exists()){
        dir.mkpath(outPath);
    }
    return a7zUncompress(inPath.toUtf8().data(),outPath.toUtf8().data());
}

int FileUtils::unCompress7zFileStd(const string& inPath,const string& outPath){
    return unCompress7zFile(QString(inPath.c_str()),QString(outPath.c_str()));
}

int FileUtils::unCompress7zFile(const QString& inPath){
    QString out = inPath.mid(0,inPath.length() - 3);
    return unCompress7zFile(inPath,out);
}

bool  FileUtils::existDir(const string& path){
    QString qpath = QString(path.c_str());
    QDir defDiskDir(qpath);
    return defDiskDir.exists();
}

bool FileUtils::mkPathIfNotExist(const string &path){
    try {
        QString qpath = QString(path.c_str());
        QDir defDiskDir(qpath);
        if(!defDiskDir.exists()){
            return defDiskDir.mkpath(qpath);
        }
    } catch (...) {
        KT_Error("mkPathIfNotExist fail : %s",path.c_str());
    }

    return true;
}

bool FileUtils::existFile(const string& path){
    QFile file(QString(path.c_str()));
    return file.exists();
}

vector<char> FileUtils::fileBytes(const string& path){
    try {
        QFile file(path.c_str());
        if(!file.exists())return vector<char>();
        if(file.open(QIODevice::ReadOnly)){
            QByteArray ba = file.readAll();
            vector<char> vec(ba.length() + 1);
            memcpy(vec.data(),ba.data(),ba.length());
            vec.push_back('\0');
            file.close();
            return vec;
        }
    } catch (...) {}
    return vector<char>();
}

vector<string> FileUtils::files(const string& dirPath){
    QDir dir(QString(dirPath.c_str()));
    if(!dir.exists())return vector<string>();
    vector<string> vec;
    for(QString& path : dir.entryList()){
        vec.push_back(path.toStdString());
    }
    return vec;
}

bool FileUtils::writeToFile(const char* data,const string& path,const string& fileName){
    mkPathIfNotExist(path);
    QString qPath = QString(path.c_str());
    QString filePath (qPath);

    if(!qPath.endsWith("/")){
        filePath.append("/");
    }
    filePath.append(fileName.c_str());
//    printf("写入 %s\n",filePath.toUtf8().data());
    try {
        QFile file(filePath);
        if(file.open(QIODevice::WriteOnly)){
            file.write(data);
            file.close();
        }else{
            KT_Error("writeToFile fail : %s",filePath.toUtf8().data());
            return false;
        }
    } catch (...) {
        KT_Error("writeToFile fail : %s",filePath.toUtf8().data());
        return false;
    }
    return true;
}

