#include "jsonutils.h"
#include "QJsonArray"
#include "QJsonObject"
#include "QJsonDocument"
#include "QJsonParseError"
#include "QDebug"
#include "exception"

JsonUtils::JsonUtils()
{

}

QVariantList JsonUtils::getArray(QByteArray& datas){
    QJsonParseError err_rpt;
    QJsonDocument root_Doc = QJsonDocument::fromJson(datas, &err_rpt);//字符串格式化为JSON
    if(err_rpt.error != QJsonParseError::NoError)//解析失败，文件被破坏
    {
        return QVariantList();
    }
    return root_Doc.array().toVariantList();
}

QVariantMap JsonUtils::getObject(QByteArray& datas){
    QJsonParseError err_rpt;
    QJsonDocument root_Doc = QJsonDocument::fromJson(datas, &err_rpt);//字符串格式化为JSON
    if(err_rpt.error != QJsonParseError::NoError)//解析失败，文件被破坏
    {
        return QVariantMap();
    }

    return root_Doc.object().toVariantMap();
}

QString JsonUtils::jsonToString(QJsonArray& v){
    return QString(QJsonDocument(v).toJson(QJsonDocument::Compact));
}

QString JsonUtils::jsonToString(QJsonObject& v){
    return QString(QJsonDocument(v).toJson(QJsonDocument::Compact));
}

QString JsonUtils::jsonToString(QJsonDocument& v){
    return QString(v.toJson(QJsonDocument::Compact));
}

QVariant JsonUtils::stringToJson(const QString& str){
    QJsonParseError l_err;
    QJsonDocument l_doc = QJsonDocument::fromJson(str.toUtf8(), &l_err);
    if (l_err.error == QJsonParseError::NoError){
        if (l_doc.isObject()){
            return l_doc.object();
        }else if(l_doc.isArray()){
            return l_doc.array();
        }
    }
    return QVariant();
}

std::string JsonUtils::mapToJsonStr(const std::map<std::string,JsonValue>& map){
    QJsonObject job;

    auto iter = map.begin();
    while(iter != map.end()){
        job[QString(iter->first.c_str())] = jsonValue2QJsonValue(iter->second);
        iter ++;
    }
    return JsonUtils::jsonToString(job).toStdString();
}

QJsonValue JsonUtils::jsonValue2QJsonValue(const JsonValue& jsonValue){
    if(jsonValue.isBoolType()){
        return QJsonValue(jsonValue.boolValue);
    }else if(jsonValue.isIntType()){
        return QJsonValue(jsonValue.intValue);
    }else if(jsonValue.isLongType()){
        return QJsonValue(jsonValue.longValue);
    }else if(jsonValue.isDoubleType()){
        return QJsonValue(jsonValue.doubleValue);
    }else if(jsonValue.isStringType()){
        return QJsonValue(QString(jsonValue.strValue.c_str()));
    }else if(jsonValue.isMapType()){
        return QJsonValue(jsonValueMap2QJsonValue(jsonValue.mapValue));
    }else if(jsonValue.isListType()){
        return QJsonValue(jsonValueList2QJsonValue(jsonValue.listValue));
    }
    return QJsonValue();
}

QJsonObject JsonUtils::jsonValueMap2QJsonValue(const map<string,JsonValue>& map){
    QJsonObject jsvalue;
    for(auto& item : map){
        jsvalue[QString(item.first.c_str())] = jsonValue2QJsonValue(JsonValue(item.second));
    }
    return jsvalue;
}

QJsonArray JsonUtils::jsonValueList2QJsonValue(const std::vector<JsonValue>& datas){
    QJsonArray jarr;
    for(auto& item : datas){
        jarr.push_back(jsonValue2QJsonValue(JsonValue(item)));
    }
    return jarr;
}

std::string JsonUtils::mapToJsonStr(const std::map<std::string,std::string>& map){
    QJsonObject job;
    auto iter = map.begin();
    while(iter != map.end()){
        job[QString(iter->first.c_str())] = QString(iter->second.c_str());
        iter ++;
    }
    return JsonUtils::jsonToString(job).toStdString();
}

//std::string JsonUtils::jsonValue2String(JsonValue& jsonValue){
//    QJsonObject job;
//    if(jsonValue.isIntType()){
//        job[jsonValue]
//    }

//    return JsonUtils::jsonToString(job).toStdString();
//}

void JsonUtils::qMap2Map(const QMap<QString,QVariant>& qmap,map<std::string,JsonValue>& map){
    auto iter = qmap.begin();
    while(iter != qmap.end()){
        QVariant value = iter.value();
        string key = iter.key().toStdString();
        auto type = value.type();
        if(type == QVariant::Type::Map){
            QMap<QString,QVariant> m = value.toMap();
            std::map<std::string,JsonValue> mv;
            qMap2Map(m,mv);
            map[key] = JsonValue(mv);

        }else if(type == QVariant::Type::List){
            QList<QVariant> list = value.toList();
            vector<JsonValue> vector;
            qList2List(list,vector);
            map[key] = JsonValue(vector);

        } else {
            map[key] = variant2JsonValue(value);
        }

        iter ++;
    }
}

QVariant JsonUtils::jsonValue2variant(const JsonValue& jsonValue){
    if(jsonValue.isBoolType()){
        return QVariant(jsonValue.boolValue);
    }else if(jsonValue.isIntType()){
        return QVariant(jsonValue.intValue);
    }else if(jsonValue.isLongType()){
        return QVariant(jsonValue.longValue);
    }else if(jsonValue.isDoubleType()){
        return QVariant(jsonValue.doubleValue);
    }else if(jsonValue.isStringType()){
        return QVariant(QString(jsonValue.strValue.c_str()));
    }else if(jsonValue.isMapType()){
        QMap<QString, QVariant> mv;
        std::map<std::string,JsonValue> m = jsonValue.mapValue;
        map2QMap(m,mv);
        return QVariant(mv);
     }else if(jsonValue.isListType()){
        QList<QVariant> v;
        std::vector<JsonValue> list = jsonValue.listValue;
        list2QList(list,v);
        return QVariant(v);
    }
    return QVariant();
}

void JsonUtils::map2QMap(const map<std::string,JsonValue>& map, QMap<QString,QVariant>& qmap){
    auto iter = map.begin();
    while (iter != map.end()) {
        QString key = QString(iter->first.c_str());
        JsonValue jsonValue = iter->second;
        if(jsonValue.isMapType()){
            std::map<string,JsonValue> mmap = jsonValue.mapValue;
            QMap<QString,QVariant> qmapv;
            map2QMap(map,qmapv);
            qmap[key] = qmapv;
        }else if(jsonValue.isListType()){
            QList<QVariant> list;
            vector<JsonValue> vector = jsonValue.listValue;
            list2QList(vector,list);
            qmap[key] = list;
        }else{
            qmap[key] = jsonValue2variant(jsonValue);
        }
        iter ++;
    }
}

void JsonUtils::list2QList(const vector<JsonValue>& list, QList<QVariant>& qlist){
    for(JsonValue item : list){
       if(item.isMapType()){
            QMap<QString, QVariant> mv;
            std::map<std::string,JsonValue> m = item.mapValue;
            map2QMap(m,mv);
            qlist.push_back(QVariant(mv));
         }else if(item.isListType()){
            QList<QVariant> v;
            std::vector<JsonValue> list = item.listValue;
            list2QList(list,v);
            qlist.push_back(QVariant(v));
        }else{
           qlist.push_back(jsonValue2variant(item));
       }
    }
}

void JsonUtils::qList2List(const QList<QVariant>& qlist,vector<JsonValue>& vector){
    for(QVariant item : qlist){
       if(item.type() == QVariant::Type::Map){
            QMap<QString, QVariant> m = item.toMap();
            std::map<std::string,JsonValue> mv;
            qMap2Map(m,mv);
            vector.emplace_back(JsonValue(mv));
         }else if(item.type() == QVariant::Type::List){
            QList<QVariant> list = item.toList();
            std::vector<JsonValue> v;
            qList2List(list,v);
            vector.emplace_back(JsonValue(v));
        }else{
           vector.emplace_back(variant2JsonValue(item));
       }
    }
}

JsonValue JsonUtils::variant2JsonValue(const QVariant& variant){
    auto type = variant.type();
    if(type == QVariant::Bool){
        bool v = variant.toBool();
        return JsonValue(v);
    }else if(type == QVariant::Type::Int){
        int v = variant.toInt();
        return JsonValue(v);
    }else if(type == QVariant::Type::LongLong){
        long long v = variant.toLongLong();
        return JsonValue(v);
    }else if(type == QVariant::Type::Double){
        double v = variant.toDouble();
        return JsonValue(v);
    }else if(type == QVariant::Type::String){
        string v = variant.toString().toStdString();
        return JsonValue(v);
    }else if(type == QVariant::Type::Map){
        QMap<QString, QVariant> m = variant.toMap();
        std::map<std::string,JsonValue> mv;
        qMap2Map(m,mv);
        return JsonValue(mv);
     }else if(type == QVariant::Type::List){
        QList<QVariant> list = variant.toList();
        std::vector<JsonValue> v;
        qList2List(list,v);
        return JsonValue(v);
    }
    return JsonValue();
}

void JsonUtils::stringToJson(const std::string& str,map<std::string,JsonValue>& out){
    QMap<QString, QVariant> data = JsonUtils::stringToJson(QString(str.c_str())).toMap();
    qMap2Map(data,out);
}
