#include "loginprivwindow.h"
#include "QIcon"
#include  "mainwindow.h"
#include <QVariant>

using namespace std;
extern CAppSetting* g_cappSetting;

LoginPriWindow::~LoginPriWindow(){
}

    LoginPriWindow::LoginPriWindow(QWidget * container,QSize& size,int windowBlurRadius):QWidget(container)
    {
        setFixedSize(size);
        this->_container = this;

        _labelFont.setPixelSize(15);
        _labelFont.setWeight(0);

        QVBoxLayout *vBox = new QVBoxLayout;

        vBox->setContentsMargins(0,0,0,0);
        vBox->setSpacing(0);
        QWidget* coverImage = addCoverImage(_container,windowBlurRadius);//顶部巨幅
        coverImage->setObjectName("coverImage");
        mCoverContainer = new QWidget();
        QHBoxLayout* containerLay = new QHBoxLayout(mCoverContainer);
        containerLay->setContentsMargins(0,0,0,0);
        containerLay->setSpacing(0);
        containerLay->addWidget(coverImage);
        vBox->addWidget(mCoverContainer);

        vBox->addSpacing(7);

        QLabel* label = new QLabel;
        QFont lFont = QFont(_labelFont);
        lFont.setPixelSize(13);
        label->setFont(lFont);
        vBox->addWidget(label);

        vBox->addWidget(accountEditLine(_container));
        vBox->addWidget(_accountLine = line(_container));

        vBox->addSpacing(16);
        vBox->addWidget(loginBtn(_container));

        vBox->addStretch();
        setLayout(vBox);

        _loginType = new QLabel(_container);
        _loginType->setText("登录-私有云");
        QFont font;

        font.setPixelSize(14);
        _loginType->setFont(font);
        QPalette palette;
        palette.setColor(QPalette::WindowText,Qt::white);
        _loginType->setPalette(palette);
       _loginType->setGeometry(10,5,100,40);

       _toast = new ToastWidget(this);

       vector<string> history = g_cappSetting->getLoginHistoryManager()->getPrivCloudAcc();
       if(history.size() > 0){
           _accountEdit->setText(QString(history[0].c_str()));
       }

       mToastHander = new MainThreadHandler(this);
       connect(mToastHander,&MainThreadHandler::task,this,[=](QVariant data){
           _toast->showMesssage(data.toString());
       });

       mCompHander = new MainThreadHandler(this);
       connect(mCompHander,&MainThreadHandler::task,this,[=](QVariant data){
           m_mainwindow->setWindowExFunButtonClickable(true);
           _progdialog->dismiss();
           RemoteDesktopManager* manager = RemoteDesktopManager::getInstance();
           if(data.toBool()){
               mIsConnectSuc = true;
               manager->startRemoteDesktop(_rdpCode);
               manager->removeConnectListener(this);
               m_mainwindow->closeMainWindow();
           }
       });
    }

    void LoginPriWindow::setRadius(int radius){
        auto child = mCoverContainer->findChild<QWidget*>("coverImage");
        QWidget* cover = addCoverImage(_container,radius);

        mCoverContainer->layout()->removeWidget(child);
        mCoverContainer->layout()->addWidget(cover);
        child->setParent(nullptr);
        child->deleteLater();
        cover->setObjectName("coverImage");
    }

    QSize LoginPriWindow::getLoginPriWindowSize(){
        return QSize(420,273);
    }

    void LoginPriWindow::onItemClick(ExpandListPop * p , QString& text,int index) {
        p->dismiss();
        if(_accountEdit){
            _accountEdit->setText(text);
        }
    }

    void LoginPriWindow::closeEvent(QCloseEvent *event){
        RemoteDesktopManager::getInstance()->removeConnectListener(this);
        if(!mIsConnectSuc && _rdpCode >-1){
            RemoteDesktopManager::getInstance()->destroyRdpCode(_rdpCode);
        }
    }

    void LoginPriWindow::setMainWindow(MainWindow *window){
        this->m_mainwindow = window;
    }

    QWidget* LoginPriWindow::line (QWidget * container){
        ImageView* split = new ImageView;
        split->setFixedSize(container->width() - _lrpadding * 2,1);
        split->setBackgroudColor(_borderColor);
        QHBoxLayout* lay = new QHBoxLayout;
        lay->setContentsMargins(0,0,0,0);
        lay->setSpacing(0);

        lay->addWidget(split);
        QWidget* line = new QWidget;
        line->setLayout(lay);
        return line;
    }

    void LoginPriWindow::onClick(ImageView *, QMouseEvent *){
       RemoteDesktopManager* manager = RemoteDesktopManager::getInstance();
       for(auto& info : manager->getConnectInfo()){
         RemoteConnectInfo cInfo = info.second;
         if(cInfo.type == RemoteConnectInfo::PRIVATE && cInfo.name == _accountEdit->text().toStdString()){
             _toast->showMesssage("该账号正在被使用");
             return;
         }
       }

       QSize size = QGuiApplication::screens()[0]->availableSize();

       _progdialog = new ProgressDialog(this);
       _progdialog->setRadius(g_cappSetting->getWindowRadius());
        _progdialog->showDialog("登录中..");
        m_mainwindow->setWindowExFunButtonClickable(false);
        manager->addConnectListener(this);
        _rdpCode = manager->loginPrivateCloud(
                    _accountEdit->text().trimmed().toStdString().c_str(),
                    g_cappSetting->getDeviceId().c_str(),
                    g_cappSetting->getSDiskPath().c_str(),
                    size.width(),
                    size.height() - style()->pixelMetric(QStyle::PM_TitleBarHeight));

        if(_rdpCode < 0){
            _toast->showMesssage("该账号正在被使用");
            _progdialog->dismiss();
        }
     }

    void LoginPriWindow::statusChanged(RdpRemoteConnectInfo& cInfo){
        int rdpCode = cInfo.rdpCode;
        int code = cInfo.status;
        QString info = QString(cInfo.msg.c_str());
        if(rdpCode != _rdpCode){
            return;
        }
        if(code > 2){
            if(code != RdpRemoteConnectInfo::Normal){
                QString retVal;
                toastMsg(info);
            }else{
                g_cappSetting->getLoginHistoryManager()->addPricloudLoginAccIfNoExist(_accountEdit->text().toStdString());
            }
            connectCompleted(info,code == RdpRemoteConnectInfo::Normal);
         }
    }
    void LoginPriWindow::remoteUIChanged(int ,uint8_t *){}
    void LoginPriWindow::toastMsg(QString& s){
        QVariant v(s);

        mToastHander->addTaskToThread(v);
    }

    void LoginPriWindow::connectCompleted(QString s,bool suc){
        QVariant res(suc);
        mCompHander->addTaskToThread(res);
    }

    QWidget * LoginPriWindow::loginBtn(QWidget * container){
        QHBoxLayout* lay = new QHBoxLayout;
        _login_btn = new ImageView;
        _login_btn->setFixedSize(container->width() - _lrpadding * 2,30);
        _login_btn->setBackgroudColor(_login_btn_disclick_bg_color);
        _login_btn->setClickColor(QColor(101,191,255,255));

        _login_btn->setLabel(QObject::tr("login"),"login");
        _login_btn->setClipRadius(5);
        _login_btn->setFontColor(Qt::white);
        _login_btn->setFont(_labelFont);
        lay->addWidget(_login_btn);
        lay->setSpacing(0);
        lay->setContentsMargins(0,5,0,0);
        QWidget *w = new QWidget;
        w->setLayout(lay);

        _login_btn->setOnClickListener(this);
        _login_btn->setClickable(false);

        return w;
    }

    QWidget* LoginPriWindow::accountEditLine(QWidget * container){
        QWidget* content = new QWidget;
        int itmHeight = 25;

        QHBoxLayout* layout = new QHBoxLayout;
        layout->setContentsMargins(_lrpadding,0,_lrpadding,0);

        layout->setSizeConstraint(QLayout::SetMaximumSize);
        layout->setSpacing(0);

        _accountEdit = new QMLineEdit;
        _accountEdit->setFont(_labelFont);

        connect(_accountEdit,&QMLineEdit::onKeyDown,this,[=](int key){
            if(key == Qt::Key_Return){
                onClick(nullptr,nullptr);
            }
        });

        connect(_accountEdit,&QLineEdit::textChanged,[=](QString str){
            bool clickable = !str.isEmpty();
            _login_btn->setClickable(clickable);
            _login_btn->setBackgroudColor(clickable ? _login_btn_bg_color : _login_btn_disclick_bg_color);
        });

        QPalette palette;
        palette.setColor(QPalette::Text, _labelColor);
        _accountEdit->setPalette(palette);
        _accountEdit->setFixedHeight(itmHeight);

        _accountEdit->setPlaceholderTextFromTr("请输入KIS产品服务器地址/ID/授权码","");
        _accountEdit->setAttribute(Qt::WA_MacShowFocusRect,false);

        _accountEdit->setStyleSheet("QLineEdit{border-width:0;border-style:outset;margin-left:5}");
        layout->addWidget(_accountEdit);

    //     layout->addWidget(comIconBtn(":/images/clear.svg"));
        ImageView * accExpBtn = comIconBtn(":/images/expand_more.svg");
        accExpBtn->setTag(this);
        accExpBtn->setClickListener([](ImageView * img,QMouseEvent*){
            LoginPriWindow * lWindow = ((LoginPriWindow*)img->getTag());

            vector<string> history = g_cappSetting->getLoginHistoryManager()->getPrivCloudAcc();
            if(history.size() == 0){
                lWindow->_toast->showMesssage("没有登录记录!");
                return;
            }

            QWidget * line = lWindow->_accountLine;
            int l = line->pos().x();
            int t = line->pos().y();
            QWidget * parent = (QWidget*)line->parent();
            while(parent != nullptr && parent->isWidgetType()){
                l += parent->pos().x();
                t += parent->pos().y();
                parent = (QWidget*)parent->parent();
            }

            ExpandListPop* p = new ExpandListPop(290+40);
            p->setItemPadding(13,0,7,0);

            QList<ExMenuButtonInfo> list;
            QList<QString> labels;
            for(string& label : history){
                ExMenuButtonInfo exInfo;
                exInfo.path = ":images/ic_clear.svg";
                exInfo.isSvg = true;
                exInfo.width = 30;
                exInfo.height = 30;
                exInfo.paddingTop = 8;
                exInfo.paddingLeft = 8;
                exInfo.paddingRight = 8;
                exInfo.paddingBottom = 8;
                list.push_back(exInfo);
                labels.push_back(QString(label.c_str()));
            }

            p->setItems(labels,list);
            p->showPop(l + lWindow->_lrpadding-5,t+1);
            p->setItemClickListener((LoginPriWindow*)img->getTag());
            connect(p,&ExpandListPop::itemMenuClick,p,[=](int index){
                vector<string> history = g_cappSetting->getLoginHistoryManager()->getPrivCloudAcc();
                if((int)history.size() > index){
                    g_cappSetting->getLoginHistoryManager()->deletePrivCloudAcc(history[index]);
                    p->removeItem(index);
                }
            });
        });
        layout->addWidget(accExpBtn);
        content->setLayout(layout);

    //    content->setFixedSize(container->width() , itmHeight);
        return content;
    }

    ImageView * LoginPriWindow::comIconBtn(const QString& path){
        ImageView*imageView = new ImageView;
        imageView->setFixedSize(35,35);
        imageView->setRadius(100);
        imageView->waveCenter(true);
        imageView->setPadding(7,7,7,7);
        imageView->setSvgPixMapSrc(path,35,35);

        return imageView;
    }

    QLabel * LoginPriWindow:: addCoverImage(QWidget * container,int windowBlurRadius){
        QLabel * converImage = new QLabel;
        QPixmap pixmap(":/images/login_banner.png");

        double pr = QGuiApplication::primaryScreen()->devicePixelRatio();

        QPixmap sp = pixmap.scaled(container->width() * pr,1,Qt::KeepAspectRatioByExpanding);
        sp.setDevicePixelRatio(pr);

        int off = sp.height() * .07;
        QPixmap map(sp.width(),sp.height() - off * 2);
        map.fill(Qt::transparent);
        map.setDevicePixelRatio(sp.devicePixelRatio());
        QPainter mapPainter(&map);
        QPainterPath path;
        mapPainter.setPen(Qt::NoPen);

        path.addRoundedRect(0,0,container->width(),sp.height() ,windowBlurRadius,windowBlurRadius);
        mapPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        mapPainter.setClipPath(path);
        mapPainter.drawPixmap(0,-off/2,sp);
        mapPainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);

        QPixmap conver(sp.width(),sp.height());
        conver.setDevicePixelRatio(pr);
        conver.fill(Qt::transparent);
        QPainter converPainter(&conver);
        converPainter.setPen(Qt::NoPen);
        converPainter.setBrush(Qt::black);
        converPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        converPainter.drawRoundedRect(0,0,container->width(),sp.height(),windowBlurRadius,windowBlurRadius);
        mapPainter.drawPixmap(0,0,conver);

        converImage->setPixmap(map);
        return converImage;
    }
;

