#pragma once
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QLabel>
#include <QGraphicsDropShadowEffect>
#include <QPainter>
#include <QPalette>
#include <QBitmap>
#include <qdesktopwidget.h>
#include "accountselectwindow.h"
#include <iostream>
#include <QGuiApplication>
#include <QScreen>
#include "loginprivwindow.h"
#include <QMouseEvent>
#include <QPoint>
#include <QTranslator>
#include "widgets/tansbyywindow.h"
#include "settingpage.h"
#include "dataManager/cppurls.h"

extern CAppSetting* g_cappSetting;
MainWindow::MainWindow(QWidget *parent,bool suportAutoLogin)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    this->mSuportAutoLogin = suportAutoLogin;

    blurRadius = g_cappSetting->getWindowRadius();
//    LangTransManager * tran = new LangTransManager;
//    tran->loadLang(LangType::ZH_TW);

    setWindowFlags(Qt::Window |Qt::FramelessWindowHint);
    if(blurRadius > 0)setAttribute(Qt::WA_TranslucentBackground);
    mShadow_effect = new QGraphicsDropShadowEffect(this);
    mShadow_effect->setOffset(0, 0);

    int shadowAlfa = 180;
    #ifdef Q_OS_MAC
     shadowAlfa = 100;
    #endif
    mShadow_effect->setColor(QColor(0,0,0,shadowAlfa));

    mShadow_effect->setBlurRadius(14);

    m_contain = new LoginContain(nullptr,blurRadius);

    setCentralWidget(m_contain);

    setContentsMargins(blurRadius+1,blurRadius+1,blurRadius,blurRadius);
    QDesktopWidget *desktop = QApplication::desktop();

    QSize size = LoginPriWindow::getLoginPriWindowSize();
    m_contain->setSize(size.width(),size.height());
    m_contain->setGraphicsEffect(mShadow_effect);
    QHBoxLayout* lay = new QHBoxLayout;
    lay->setContentsMargins(0,0,0,0);
    lay->setSpacing(0);

    QSize s = QSize(size.width(),size.height());
    mLoginPriPage = new LoginPriWindow(nullptr,s,blurRadius);
    mLoginPriPage->setMainWindow(this);

    QSize lsize = LoginWindow::windowSize();
    QSize loginWindowSize = QSize(lsize.width(),lsize.height());
    mQrWindow = new QrLoginWindow(nullptr,QrLoginWindow::getWindowSize());
    mQrWindow->setMainWindow(this);

    mLoginPage = new LoginWindow(nullptr,loginWindowSize,blurRadius);
    mLoginPage->setMainWindow(this);

    mAccSelectWindow = new AccountSelectWindow(nullptr,loginWindowSize,blurRadius);
    mAccSelectWindow->setMainWindow(this);

    mAboutWindow = new AboutWindow(nullptr,loginWindowSize,blurRadius);
    mAboutWindow->setMainWindow(this);

    lay->addWidget(mLoginPriPage);
    lay->addWidget(mQrWindow);
    lay->addWidget(mLoginPage);
    lay->addWidget(mAccSelectWindow);

    m_contain->setLayout(lay);
    mQrWindow->setVisible(false);
    mAccSelectWindow->setVisible(false);

    setShowMode(g_cappSetting->isLoginPrivCloudMode() ? 2 : 1);

    QWidget* p = optButtons(m_contain);
    p->setGeometry(0,-9,p->width(),p->height());

    lay->addWidget(mAboutWindow);
    mAboutWindow->setVisible(false);

    move((desktop->width() - width())/ 2, (desktop->height() - height()) /2);
    setWindowIcon(QIcon(":/images/icon_lauch.png"));
    setWindowTitle("登录");
}

void MainWindow::boundGraphicsEffectEnable(bool enable){
    m_contain->setGraphicsEffect(enable ? mShadow_effect : nullptr);
}

void MainWindow::setTansByYWindow(TansByYWindow* w){
    this->mHostWindow = w;
}

void MainWindow::changeEvent(QEvent * e) {

}

void MainWindow::popSelected(PopupMenuWindow* pop,int index){
    pop->dismiss();
    bool isPri = g_cappSetting->isLoginPrivCloudMode();
    if(index > 0)index = isPri ? index + 2 : index;
    switch (index) {
    case 0:
        g_cappSetting->setLoginPrivCloudMode(!g_cappSetting->isLoginPrivCloudMode());
        setShowMode(g_cappSetting->isLoginPrivCloudMode() ? 2 : 1);
        if(!g_cappSetting->isLoginPrivCloudMode()){
            mLoginPage->changeLoginTypeLabel();
        }
        break;
    case 2:
        g_cappSetting->setExpMode(!g_cappSetting->isExpMode());
        mLoginPage->changeLoginTypeLabel();
        break;

    case 4:
        toSetting();
        break;

    case 6:
        AppUtils::openUrlByDefaultBrower(CppUrls::getUrlLogUpdate());
        break;
    case 7:
        AppUtils::openUrlByDefaultBrower(CppUrls::getUrlHelpDoc());
        break;
    case 8:
        setShowMode(5);
        break;
    }
}

void MainWindow::setWindowRoundable(bool enable){
    hide();
    int radius = g_cappSetting->getRadiusSize();
    int targetRadius = enable ? radius : 0;

    int padding = enable ?targetRadius:1;

    setContentsMargins(padding,padding,0,0);

    m_contain->setRadius(targetRadius);
    mLoginPage->setRadius(targetRadius);
    mLoginPriPage->setRadius(targetRadius);
    mAccSelectWindow->setRadius(targetRadius);
    g_cappSetting->setWindowRadius(targetRadius);
    setShowMode(mShowMode);
    show();
}

void MainWindow::toSetting(){
    SettingPage* window = new SettingPage;

    QPoint point = pos();
    int x = point.x();
    int y = point.y();
    int showX ;
    if(x < window->width()*1.5){
        showX = x + width() - blurRadius+2;
    }else{
        showX = x - window->width()+blurRadius-2;
    }
    window->show(showX,y + blurRadius);

}

void MainWindow::funClick(ImageView* img, QMouseEvent * e){
    PopupMenuWindow * w = new PopupMenuWindow;
    QFont font;
    font.setPixelSize(10);

    bool isPri = g_cappSetting->isLoginPrivCloudMode();
    w->addItem(isPri?"公有云模式":"私有云模式");
    w->addLine();
    if(!isPri){
        w->addItem("体验版",g_cappSetting->isExpMode());
        w->addLine();
//        w->addItem("忘记密码");
    }

    w->addItem("高级设置");

    w->addLine();
    w->addItem("更新日志");
    w->addItem("帮助手册");
    w->addItem("关于");

    w->setFixedWidth(175);

    w->setSelectItemLisenter(this);

    w->showPopup(e->globalX(),e->globalY());

}

void MainWindow::onClick(ImageView * image, QMouseEvent * e){
    funClick(image,e);
}

QWidget* MainWindow::optButtons(QWidget * container){
    QHBoxLayout * layout = new QHBoxLayout;
//    layout->setContentsMargins(0,0,0,0);

    layout->setAlignment(Qt::AlignHCenter|Qt::AlignRight);

    QSize size(21,21);
    m_exFunBtn = new ImageView();
    m_exFunBtn->setSign("showFunBtn");
    m_exFunBtn->setFixedSize(36,36);
    m_exFunBtn->setScaleType(IMVScaleType::fillWH);
    m_exFunBtn->setPadding(10,10,10,10);
    m_exFunBtn->setRadius(40);
    m_exFunBtn->waveCenter(true);
//    exFunBtn->setTag(this);
    m_exFunBtn->setPixMapSrc(QPixmap(":/images/downarrow.png"));
    m_exFunBtn->setOnClickListener(this);

    layout->addWidget(m_exFunBtn);

    minBtn = new WindowMinButton;
    minBtn->setHost(this);
    minBtn->setFixedSize(size);
    layout->addWidget(minBtn);
    layout->addSpacing(5);

    mCloseButton = new WindowCloseButton();
    mCloseButton->setHost(this);
    mCloseButton->setFixedSize(size);
    layout->addWidget(mCloseButton);
    layout->addSpacing(5);

    QWidget* w = new QWidget(container);
    w->setFixedHeight(72);
    w->setFixedWidth(container->width());
    w->setLayout(layout);

    return w;
}

void MainWindow::setWindowExFunButtonClickable(bool en){
    m_exFunBtn->setClickable(en);
}

void MainWindow::setShowMode(int m){
    mLoginPriPage->setVisible(m == 2);
    mLoginPage->setVisible(m == 1);
    mQrWindow->setVisible(m == 3);
    mAccSelectWindow->setVisible(m == 4);
    mAboutWindow->setVisible(m == 5);

    int radius = g_cappSetting->getWindowRadius();

    if(m_exFunBtn)m_exFunBtn->setVisible(m !=4);
    if(m == 2){
        QSize size = LoginPriWindow::getLoginPriWindowSize();
        m_contain->setSize(size.width(),size.height());
        mLoginPriPage->setFixedSize(m_contain->size());
        setFixedSize(size.width()+radius*2+2,size.height()+radius*2+2);
     }else if(m == 1){
        QSize size = LoginWindow::windowSize();
        m_contain->setSize(size.width(),size.height());
        mLoginPage->setFixedSize(m_contain->size());
        setFixedSize(size.width()+radius*2+2,size.height()+radius*2+2);
     }else if(m == 3){
        QSize size = QrLoginWindow::getWindowSize();
        m_contain->setSize(size.width(),size.height());
        mLoginPage->setFixedSize(m_contain->size());
        setFixedSize(size.width()+radius*2+2,size.height()+radius*2+2);
        mQrWindow->loadQrData();
    }else if(m == 5){
        mAboutWindow->setLastMainMode(mShowMode);
        QSize size = AboutWindow::getWindowSize();
        m_contain->setSize(size.width(),size.height());
        mAboutWindow->setFixedSize(m_contain->size());
        setFixedSize(size.width()+radius*2+2,size.height()+radius*2+2);
    }else if(m == 4){
        QSize size = AccountSelectWindow::windowSize();
        m_contain->setSize(size.width(),size.height());
        mAccSelectWindow->setFixedSize(m_contain->size());
        setFixedSize(size.width()+radius*2+2,size.height()+radius*2+2);
    }
    mShowMode = m;
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *event){
    mLoginPriPage->closeEvent(event);
    mAccSelectWindow->closeEvent(event);
}

void MainWindow::mousePressEvent(QMouseEvent *e){

    if(e->button() == Qt::LeftButton){
        _canMoveWindow = true;
        mMovePosition = e->globalPos() - this->pos();
    }
}

void MainWindow::mouseMoveEvent(QMouseEvent *event){
    if( _canMoveWindow){
        this->move(event->globalPos() - mMovePosition);
        mMovePosition = event->globalPos() - this->pos();
    }
}

void MainWindow::mouseReleaseEvent(QMouseEvent *event){
    if(event->button() == Qt::LeftButton){
        _canMoveWindow = false;
    }
}

void MainWindow::leaveEvent(QEvent *){
    _canMoveWindow = false;
}

QPoint MainWindow::mainWindowPos(){
    return this->pos();
}

QSize MainWindow::mainWindowSize(){
   return this->size();
}

void MainWindow::setMainWindowFixSize(QSize size){
    this->setFixedSize(size);
    LoginContain* contain = (LoginContain*)this->centralWidget();
    contain->setSize(size.width(),size.height());
}

void MainWindow::closeMainWindow(){
    this->close();
    #ifndef Q_OS_MAC
        deleteLater();
    #endif
}
