#include "navstringutils.h"
#include <QByteArray>
#include <QCryptographicHash>
#include <QUuid>
#include <QRegExp>
#include <utils/qaesencryption.h>
#include <iconv.h>
#include <QUrl>

NavStringUtils::NavStringUtils()
{

}
std::string NavStringUtils::md5(const std::string& dataStr){
    QByteArray data = QCryptographicHash::hash ( dataStr.c_str(), QCryptographicHash::Md5 );
    return std::string(data.toHex());
}

std::string NavStringUtils::getUUid(){
    QUuid uuid = QUuid::createUuid();
    QString uuidStr = uuid.toString();
    uuidStr.remove(0,1);
    uuidStr.remove(uuidStr.count() - 1 , 1);
    uuidStr.remove(QRegExp("-"));
    return std::string(uuidStr.toUtf8().data());
}

std::string NavStringUtils::aesCBCPKCS5PADDINGEncode(const std::string& pcInput,const std::string& key, const std::string& iv){
    std::string str = std::string(aesCBCPKCS5PADDINGEncode2Byte(pcInput,key,iv));
    return str;
}

std::string NavStringUtils::aesCBCPKCS5PADDINGEncode2Byte(const std::string& pcInput,const std::string& key, const std::string& iv){
    QAESEncryption encryption(QAESEncryption::AES_128, QAESEncryption::CBC,QAESEncryption::PKCS7);
    QByteArray encodedText = encryption.encode(pcInput.c_str(),key.c_str(),iv.c_str());
    return std::string(encodedText.toBase64().data());
}

std::string NavStringUtils::aesCBCPKCS5PADDINGDecode(const std::string& pcInput,const std::string& key, const std::string& iv){

    QAESEncryption encryption(QAESEncryption::AES_128, QAESEncryption::CBC,QAESEncryption::PKCS7);
    QByteArray dataDecode = QByteArray::fromBase64(pcInput.c_str());

    QByteArray decodeData = encryption.decode(dataDecode,key.c_str(),iv.c_str());
    int size = decodeData.length();
    if(size == 0)return "";
    int removeNum = (int)decodeData[size - 1];
    return std::string(decodeData.remove(size - removeNum , removeNum).data());
}

std::string NavStringUtils::base64QString(const std::string& encrypt_data){
    return std::string(QString(encrypt_data.c_str()).toUtf8().toBase64().data());
}

std::string NavStringUtils::urlEncode(const std::string&url){
    return QUrl::toPercentEncoding(QString(url.c_str())).toStdString();
}

bool NavStringUtils::is_base64(const char c){
    return (isalnum(c) || (c == '+') || (c == '/'));
}

bool NavStringUtils::gbk2utf8(char* src, char* dest, size_t inlen)
{
    char *inbuf = src;
    size_t outlen = inlen * 4;
    char *outbuf = dest;
//    memset(outbuf, 0, outlen);
    iconv_t cd = iconv_open("UTF-8", "GBK");
    /* iconv接口会改变inbuf和outbuf，所以不能直接使用src和dest */
    size_t res = iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
   iconv_close(cd);
    return (0 == res) ? true : false;
}

std::string NavStringUtils::gbk2Utf8String(char* src,size_t inlen){
    const int len = inlen;
    char outbuf[len];
    memset(outbuf,0,len);
    size_t z = inlen;
    gbk2utf8(src,outbuf,z);
    return std::string(outbuf);
}


