#include "qrloginwindow.h"
#include "QLabel"
#include "widgets/mouseinoutevwidget.h"
#include "widgets/imageview.h"
#include "mainwindow.h"
#include "QPalette"
#include "QPushButton"
#include "utils/AppUtils.h"
#include "utils/pixmaputils.h"
#include "widgets/qrlabel.h"
#include "presenters/qrloginpresenter.h"
#include "presenters/accountpresenter.h"
//#include "dataManager/net/netmanager.h"
//#include "dataManager/net/nettemplete.h"
#include "pojo/enterpriseaccinfo.h"
#include "accountselectwindow.h"
#include "dataManager/cappsetting.h"
#include "manager/rdpmanager.h"

extern CAppSetting* g_cappSetting;

QrLoginWindow::QrLoginWindow(QWidget *parent,QSize size) : QWidget(parent)
{
    mSize = size;
    _windowBlurRadius = g_cappSetting->getWindowRadius();
    mPresenterPtr = make_unique<QrLoginPresenter>();
    mPresenterPtr.get()->setQrCodeResponseListener([&](const string& content,int code){
        if(code != 200){
            mQrLabel->setProgress(true,code == -1 ? "网络错误" : "加载失败");
        }else{
            QPixmap map = PixmapUtils::createQRCode(QString(content.c_str()),mQrWidth);
            mQrLabel->setImage(map);
            mQrLabel->setProgress(false,"");
        }
    });

    mPresenterPtr.get()->setRespQrCodeStatusListener([&](int status){
        if(status == 200){//登录成功
            _progdialog = new ProgressDialog(this);
            _progdialog->setRadius(g_cappSetting->getWindowRadius());
            _progdialog->showDialog("登录中..");
            mPresenterPtr.get()->login([&](bool suc,bool finished,const std::string& msg){
                if(suc){
                    if(!finished){
                        _progdialog->setLabel(QString(msg.c_str()));
                    }else{
                        goSelectAccWindow(mPresenterPtr.get()->getUid(),mPresenterPtr.get()->getToken());
                    }
                }else{
                    _progdialog->dismiss();
                    _toast->showMesssage(QString(msg.c_str()));
                }
            },[&](ReponseResult<RdpConnectResult>& connectResult){
                if(!connectResult.isSuc()){
                    _progdialog->dismiss();
                    if(mPresenterPtr.get()->getLoginenterpriseInfos().size() > 1){
                         goSelectAccWindow(mPresenterPtr.get()->getUid(),mPresenterPtr.get()->getToken());
                    }else{
                         _toast->showMesssage(QString(connectResult.getErrMsg().c_str()));
                    }
                 }else{
                    RdpConnectResult connRes = connectResult.getData();
                    if(!connRes.isFinished){
                        _progdialog->setLabel(QString(connRes.msg.c_str()));
                    }else{
                        _progdialog->dismiss();
                        if(connRes.isWaring){
                            AlertDialog* alert = new AlertDialog(this);

                            alert->setFixedSize(this->size());
                            alert->setRadius(_windowBlurRadius);
                            vector<QString> btns ;
                            if(connRes.canVisAcc){
                                btns = {"取消","确定"};
                            }else{
                                btns = {"确定"};
                            }
                            alert->setButtons(btns);
                            alert->setTitle("提示");
                            alert->setMessage(QString(connRes.msg.c_str()));
                            connect(alert,&AlertDialog::buttonClick,this,[=](int index){
                                alert->dismiss();
                                alert->setParent(nullptr);
                                alert->deleteLater();
                                if(index == 1){
                                    _progdialog = new ProgressDialog(this);
                                    _progdialog->setRadius(g_cappSetting->getWindowRadius());
                                     _progdialog->showDialog("进入账套中..");
                                    mPresenterPtr.get()->enterAcc();
                                }
                            });

                            alert->show();
                        }else{
                            _progdialog->dismiss();
                            if(connRes.success){
                                 mHost->closeMainWindow();
                            }else{
                                _toast->showMesssage(QString(connRes.msg.c_str()));
                            }
                        }
                    }
                }
            });
        }else if(status == 207){//超时
            mQrLabel->setProgress(true,"二维码已失效");
        }else if(status == 208){//扫码成功，等待确认
            mQrLabel->scanSuccess();
        }
    });

    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->setAlignment(Qt::AlignHCenter);
    layout->setContentsMargins(20,mPaddingTop,20,10);
    layout->setSpacing(0);
    layQrLay();
    layout->addSpacing(mQrWidth+15);
    layout->addLayout(getDecLay());
    layout->addSpacing(23);
    layout->addWidget(getBackButton());
    layout->addStretch();

    _toast = new ToastWidget(this);
}

void QrLoginWindow::setMainWindow(MainWindow * m){
    this->mHost = m;
}

void QrLoginWindow::goSelectAccWindow(const string& uid,const string& token){
    vector<EnterpriseAccInfo*> vec = mPresenterPtr.get()->getLoginenterpriseInfos();
    _progdialog->dismiss();
    mHost->getAccountSelectWindow()->setData("","",vec,QString(uid.c_str()),QString(token.c_str()));
    mHost->setShowMode(4);
}

QHBoxLayout* QrLoginWindow::layQrLay(){
    int margin = 20;
    QPixmap pmap = QPixmap(":images/nav_scan.png");

    mQrLabel = new QrLabel(this);
    QPixmap map = PixmapUtils::createQRCode("rfdsewfdfdsa1231fdsfdsaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",pmap.width());
    mQrLabel->setImage(map);

    mQrWidth = pmap.width();

    const int startX = (mSize.width() - pmap.width() *2 - margin) / 2;
    const int endX = mSize.width() - startX;

    mQrLabel->setGeometry(startX,mPaddingTop,endX - startX,pmap.height());
    int moveX = (endX - startX - mQrWidth) / 2;
    mQrLabel->moveImage(moveX);
//    qrLabel->setFixedSize(pmap.size());

    mAnimTimeLine = new QTimeLine(150,this);
    mAnimTimeLine->setFrameRange(moveX , 0);
    connect(mAnimTimeLine,&QTimeLine::frameChanged,this,[=](int frame){
        mQrLabel->moveImage(frame);
    });

    connect(mAnimTimeLine,&QTimeLine::finished,this,[=](){
        if(mMouseIsEnter){
            if(mAnimTimeLine->direction() != QTimeLine::Forward){
                 mAnimTimeLine->setDirection(QTimeLine::Forward);
                 mAnimTimeLine->start();
                 mQrNavLabel->setVisible(true);
            }
        }else if(mAnimTimeLine->direction() != QTimeLine::Backward){
            mAnimTimeLine->setDirection(QTimeLine::Backward);
            mAnimTimeLine->start();
            mQrNavLabel->setVisible(false);
       }
    });

    mAnimTimeLine->setDirection(QTimeLine::Backward);

    mQrNavLabel = new QLabel(this);
    mQrNavLabel->setVisible(false);
    mQrNavLabel->setPixmap(pmap);
    mQrNavLabel->setGeometry(mSize.width() - startX - pmap.width(),mPaddingTop,pmap.width(),pmap.height());

    MouseInOutEvWidget* m = new MouseInOutEvWidget(this);

    connect(m,&MouseInOutEvWidget::clicked,this,[=](QPoint& p){
        int qrImageX = mQrLabel->getImageX();
        if(p.x() >= qrImageX && p.x() <= qrImageX +mQrWidth)
            loadQrData();
    });

    m->setGeometry(startX,mPaddingTop,endX - startX,pmap.height());
    connect(m , & MouseInOutEvWidget::mouseEvent,this,[=](bool enter){
        mMouseIsEnter = enter;
        if(mAnimTimeLine->state() == QTimeLine::Running)return;
        if(mQrImageX == -1){
            mQrImageX = mQrLabel->x();
        }

        auto dir = enter ? QTimeLine::Forward : QTimeLine::Backward;
        if(mAnimTimeLine->direction() == dir)return;
        mAnimTimeLine->setDirection(dir);
        mAnimTimeLine->start();
        mQrNavLabel->setVisible(mAnimTimeLine->direction() == QTimeLine::Forward);
    });
    return nullptr;
}

QHBoxLayout * QrLoginWindow::getDecLay(){
    QHBoxLayout * decLay = new QHBoxLayout;
    decLay->setContentsMargins(0,0,0,0);
    decLay->setSpacing(0);
    decLay->setAlignment(Qt::AlignCenter);

    QLabel* label1 = new QLabel;
    label1->setText("请使用");
    decLay->addWidget(label1);

    QPushButton* label2 = new QPushButton;
//    label2->setStyleSheet("color:rgb(56,123,246);border:none");
    label2->setStyleSheet("QPushButton{color:rgb(56,123,246);border:none}"
                          "QPushButton:hover{text-decoration:underline}");

    connect(label2,&QPushButton::clicked,this,[=]{
        AppUtils::openUrlByDefaultBrower(QString("https://kisyun.kingdee.com"));
    });

    label2->setText("KIS云APP");
    decLay->addWidget(label2);

    QLabel* label3 = new QLabel;
    label3->setText("扫码登录");
    decLay->addWidget(label3);

    return decLay;
}

QWidget* QrLoginWindow::getBackButton(){
    ImageView * btn = new ImageView;
    btn->setFixedSize(mSize.width() - 160,29);
    btn->setBackgroudColor(QColor(91,201,251,150));
    btn->setLabel("返回","login");
    btn->setClipRadius(5);
    btn->setFontColor(Qt::white);
    connect(btn,&ImageView::clicked,this,[=](){
        mHost->setShowMode(1);
        mPresenterPtr.get()->pause();
    });
    return btn;
//    btn->setFont(_labelFont);
}

void QrLoginWindow::loadQrData(){
    qDebug() << "loadQrData before";
    if(mPresenterPtr.get())mPresenterPtr.get()->getQrCode();
    qDebug() << "loadQrData after";
    mQrLabel->setProgress(true,"加载中..");
}
