#include "remotedesktopwindow.h"
#include "QPainter"
#include "QMouseEvent"
#include "rdp/rdpwindowmanager.h"
#include <immintrin.h>
#include <QMenuBar>
#include "widgets/event/dragevent.h"
#include <QMouseEvent>
#include "dialog/platetoclosepage.h"
#include <QPoint>
//#include "utils/commonlog.h"
#include <utils/deviceutils.h>

void RemoteDesktopWindow::FocusLineEdit::setKeyEventListener(FocusLineEditKeyEvent* e){
    this->mListener = e;
}

void RemoteDesktopWindow::FocusLineEdit::keyPressEvent(QKeyEvent * e){
    QLineEdit::keyPressEvent(e);
    if(mListener){
        mListener->fKeyPressEvent(e);
    }
}

void RemoteDesktopWindow::FocusLineEdit::keyReleaseEvent(QKeyEvent *e){
    QLineEdit::keyReleaseEvent(e);
    if(mListener){
        mListener->fKeyReleaseEvent(e);
    }
}

void RemoteDesktopWindow::focusInEvent(QFocusEvent *) {}

void RemoteDesktopWindow::focusOutEvent(QFocusEvent *) {}
#include <QProxyStyle>
class CustomLineEditProxyStyle : public QProxyStyle
{
public:
    virtual int pixelMetric(PixelMetric metric, const QStyleOption* option = 0, const QWidget* widget = 0) const
    {
        if (metric == QStyle::PM_TextCursorWidth)
            return 0;

        return QProxyStyle::pixelMetric(metric, option, widget);
    }
};

RemoteDesktopWindow::RemoteDesktopWindow(QWidget *parent) : QMainWindow(parent)
{
    _uithread = QThread::currentThread();
    mStatus = ConnectSuc;
    setMouseTracking(true);
    mEdit = new FocusLineEdit(this);

    mEdit->setAttribute(Qt::WA_MacShowFocusRect, 0);
//    mEdit->setStyle(new CustomLineEditProxyStyle());
//    mEdit->setStyleSheet("QLineEdit{background-color:transparent}"
//                         "QLineEdit{border-width:0;border-style:outset}");



    mEdit->setStyleSheet("QLineEdit{background-color:transparent;}"
                         "QLineEdit{border-width:0px;border-style:outset;}"
                         "QLineEdit:focus{border: 0px solid red;outline:none;}");

    mEdit->setKeyEventListener(this);
    mEdit->setFixedSize(1,1);
    connect(mEdit,&QLineEdit::textChanged,this,[=](QString text){
        int len = text.length();
        QChar* c = text.data();
        vector<ushort> unicodes;
        for(int i=0;i<len;i++){
            unicodes.push_back(c[i].unicode());
        }
        this->_manager->textChanged(unicodes,text.toStdString());
        mEdit->setText("");
    });
    mAlertDialog = new AlertDialog(this,0,0,0,100);
    mAlertDialog->setTitle("提示");
    mAlertDialog->setOnButtonClickListener(this);

    setWindowFlags(Qt::Window );
    setAttribute(Qt::WA_SetWindowModality);

    QImage bit(":/images/pop_open.png");
    bit = bit.scaledToWidth(45,Qt::SmoothTransformation);
    _menuBtn = new ClickedLabel(this);
    _menuBtn->setPixmap(QPixmap::fromImage(bit));
    _menuBtn->setFixedSize(bit.width(),bit.height());
    connect(_menuBtn,&ClickedLabel::clickedPos,this,[=](QPoint local,QPoint global){
        _manager->showMenu(global.x(),local.y());
    });

    QSize size = QSize(300,160);
    _plateToClosePage = new PlateToClosePage(size,this);
    _plateToClosePage->setHost(this);

}

void RemoteDesktopWindow::setDragEvent(DragEvent* e){
    this->mDragEvent = e;
}

void RemoteDesktopWindow::setDragEventOffset(QPoint* point){
    this->mDragEventOffset = point;
}

void RemoteDesktopWindow::onButtonCliecked(int index){
    mAlertDialog->dismiss();
    if(index == 0){
        close();
    }else{
        _manager->tryConnectAgain();
    }
}

void RemoteDesktopWindow::setManager(RdpWindowManager *manager){
    this->_manager = manager;
}

void RemoteDesktopWindow::paintEvent(QPaintEvent* event){
    if(!_image.bits() || _image.isNull() || _image.width() == 0)return;
    QPainter painter(this);
    int w = width();
    int h = height();
    painter.setRenderHints(QPainter::SmoothPixmapTransform | QPainter::Antialiasing);
    _wscale = w * 1.0 / _image.width();
    _hscale = h * 1.0 / _image.height();

    QMatrix matrix;
    matrix.scale(_wscale,_hscale);
    painter.setWorldMatrix(matrix);
//    painter.save();
    painter.fillRect(rect(),Qt::white);
//    qDebug() << "_imagebits : " << _image.bits();
    painter.drawImage(0,0,_image);

    if(mStatus == reConnecting){
        painter.setBrush(QColor(0,0,0,220));
        int pW = 57 ;
        int pH = pW ;
        QRect rect((width() - pW) / 2 , (height() - pH) / 2 , pW, pH);
        painter.setWorldMatrixEnabled(false);

        QRect progDiaRect(rect.left(),rect.top() - 100 * _hscale,rect.width(),rect.height());
        painter.drawEllipse(progDiaRect);
        painter.setPen(Qt::white);
        painter.drawText(progDiaRect,Qt::AlignCenter,"KIS");

        int off = 3;
        QRect progRect(progDiaRect.left() + off,progDiaRect.top() + off,progDiaRect.width() - off * 2,progDiaRect.height() - off * 2);

        painter.setBrush(Qt::NoBrush);

        QPen pen;
        pen.setWidthF(2);
        pen.setColor(QColor(190,190,190,190));
        painter.setPen(pen);

        painter.drawEllipse(progRect);

        pen.setColor(Qt::white);
        painter.setPen(pen);
        painter.drawArc(progRect, 76 * 16 + mConnectingProg,24*16);
    }

//    painter.restore();
    painter.end();

}

void RemoteDesktopWindow::setImageData(int w,int h,int bytePerIpxel,uint8_t* data){
//    #ifdef Q_OS_MAC
    int size = w*h*bytePerIpxel;
    if(bytePerIpxel == 4)
        for(int i = 3;i < size ; i += 4){
            *(data+i) = 255;
        }
//    #endif

    if(data != _image.bits()|| w != _image.width() || h != _image.height()){
        _image = QImage(data,w,h,QImage::Format_RGB32);
    }
    update();
}

void RemoteDesktopWindow::mouseMoveEvent(QMouseEvent *event){
    if(_manager)_manager->mouseMoveEvent(event->x() / _wscale,event->y() / _hscale);
    if(mDragEvent){
        if(mDragEventOffset){
            QMouseEvent newEvent(event->type(),event->localPos()+*mDragEventOffset,event->button(),event->buttons(),event->modifiers());
            event = &newEvent;
        }
        mDragEvent->mouseMoveEvent(event);
    }
}
void RemoteDesktopWindow::mousePressEvent(QMouseEvent *event) {
    int x = event->x() / _wscale;
    int y = event->y() / _hscale;
    if(event->button() == Qt::LeftButton){
       _manager->mouseLeftPresssEvent(x,y);
    }else if(event->button() == Qt::RightButton){
        _manager->mouseRightPresssEvent(x,y);
    }

    if(mDragEvent){
        if(mDragEventOffset){
            QMouseEvent newEvent(event->type(),event->localPos()+*mDragEventOffset,event->button(),event->buttons(),event->modifiers());
            event = &newEvent;
        }
        mDragEvent->mousePressEvent(event);
    }
}
void RemoteDesktopWindow::mouseReleaseEvent(QMouseEvent *event){
    int x = event->x() / _wscale;
    int y = event->y() / _hscale;
    if(event->button() == Qt::LeftButton){
        _manager->mouseLeftReleaseEvent(x,y);
    }else if(event->button() == Qt::RightButton){
        _manager->mouseRightReleaseEvent(x,y);
    }
    if(mDragEventOffset){
        event->pos().setX(event->pos().x() + mDragEventOffset->x());
        event->pos().setY(event->pos().y() + mDragEventOffset->y());
    }
    if(mDragEvent){
        if(mDragEventOffset){
            QMouseEvent newEvent(event->type(),event->localPos()+*mDragEventOffset,event->button(),event->buttons(),event->modifiers());
            event = &newEvent;
        }
        mDragEvent->mouseReleaseEvent(event);
    }
 };

void RemoteDesktopWindow::closeEvent(QCloseEvent *){
    _manager->windowClosed();
}

void RemoteDesktopWindow::wheelEvent(QWheelEvent *event){
   _manager->mouseScrollEvent(event->delta());
}

void RemoteDesktopWindow::fKeyPressEvent(QKeyEvent *event){
    _manager->keyPressedEvent(event->key());
}
void RemoteDesktopWindow::fKeyReleaseEvent(QKeyEvent *event){
    _manager->keyReleaseEvent(event->key());
}

void RemoteDesktopWindow::setFocusPosition(const int x ,const int y){
    if(x < 0 || y < 0)return;
    int left = x * _wscale;
    int top = y * _hscale;
    _focus_x_percent = left * 1.0 / width();
    _focus_y_percent = top * 1.0 / height();
    mEdit->setGeometry(left,top,mEdit->width(),mEdit->height());
}

void RemoteDesktopWindow::resizeEvent(QResizeEvent *){
    _manager->closeAllConver();
    _plateToClosePage->layPosition();
    if(_focus_x_percent > 0 ){
        mEdit->setGeometry(_focus_x_percent*width(),_focus_y_percent*height(),mEdit->width(),mEdit->height());
    }
    _menuBtn->setGeometry((width()-_menuBtn->width()) / 2,-_menuBtn->height()*0.17 ,_menuBtn->width(),_menuBtn->height());
    mAlertDialog->setFixedSize(width(),height());
}

void RemoteDesktopWindow::refresh(){
    repaint();
}

void RemoteDesktopWindow::setStatus(Status status){
    if(status == this->mStatus)return;
    mStatus = status;
    if(status == reConnecting){
        if(_uithread == QThread::currentThread()){
            startConnectingAnim();
        }else
            QMetaObject::invokeMethod(this, "startConnectingAnim", Qt::QueuedConnection);
    }else{
        if(_uithread == QThread::currentThread()){
            stopConnectingAnim();
        }else
            QMetaObject::invokeMethod(this, "stopConnectingAnim", Qt::QueuedConnection);
    }
}

void RemoteDesktopWindow::showErrorDialogToReTry(const string& msg){
    QList<QString> btnTexts = {"取消","重连"};
    QList<QString> canNotReConnectBtnTexts = {"确认"};
    QMetaObject::invokeMethod(this,
                              "showDialogOnMainThread",
                              Qt::QueuedConnection,
                              Q_ARG(QList<QString>,_canReConnect ? btnTexts : canNotReConnectBtnTexts),
                               Q_ARG(QString, QString(msg.c_str()))
                              );
}

bool RemoteDesktopWindow::isShowErrorDialog(){
    return !mAlertDialog->isHidden();
}

void RemoteDesktopWindow::dismissDialog(){
    QMetaObject::invokeMethod(this,"dismissDialogOnMainThread",Qt::QueuedConnection);
}

void RemoteDesktopWindow::dismissDialogOnMainThread(){
    mAlertDialog->dismiss();
}

void RemoteDesktopWindow::showErrorDialogToClose(const string& msg){
    QList<QString> list = {"关闭"};
    QMetaObject::invokeMethod(this,
                              "showDialogOnMainThread",
                               Qt::QueuedConnection,
                               Q_ARG(QList<QString>, list),
                               Q_ARG(QString, QString(msg.c_str()))
                              );
}

void RemoteDesktopWindow::showDialogOnMainThread(QList<QString> btnTexts,const QString& msg){
    std::vector<QString> vec;
    for(QString& text : btnTexts){
        vec.push_back(text);
    }
    mAlertDialog->setButtons(vec);
    mAlertDialog->setMessage(msg);
    mAlertDialog->show();
}

void RemoteDesktopWindow::stopConnectingAnim(){
    if(mProgTimeLine) mProgTimeLine->stop();
    update();
}

void RemoteDesktopWindow::startConnectingAnim(){
    mConnectingProg = 0;
    if(mProgTimeLine){
        mProgTimeLine->stop();
    }
    mProgTimeLine = new QTimeLine(1500,this);
    mProgTimeLine->setLoopCount(0);
    mProgTimeLine->setFrameRange(0,- 360 * 16);
    connect(mProgTimeLine,&QTimeLine::frameChanged,this,[=](int frame){
        mConnectingProg = frame;
        update();
    });
    mProgTimeLine->start();
}

RemoteDesktopWindow::~RemoteDesktopWindow(){
    qDebug() << "RemoteDesktopWindow deleted";
}

void RemoteDesktopWindow::countDownToClosePageOnMain(){
    if(_hasPlateToClosePage)return;
    _hasPlateToClosePage = true;
    QMetaObject::invokeMethod(this,
                              "countDownToClosePage",
                               Qt::QueuedConnection
                              );
}

void RemoteDesktopWindow::countDownToClosePage(){
    _plateToClosePage->showAnim();
}

