#ifndef REMOTEDESKTOPWINDOW_H
#define REMOTEDESKTOPWINDOW_H
#include <iostream>
#include <QMainWindow>
#include "QThread"
#include "QLineEdit"
#include "widgets/alertdialog.h"
//#include "rdp/rdpwindowmanager.h"
#include "QTimeLine"
#include "widgets/clickedlabel.h"
#include "QDebug"
#include "dialog/platetoclosepage.h"

using namespace std;
class RdpWindowManager;

class FocusLineEditKeyEvent{
public:
    virtual void fKeyPressEvent(QKeyEvent *) = 0;
    virtual void fKeyReleaseEvent(QKeyEvent *) = 0;
};
class DragEvent;

class RemoteDesktopWindow : public QMainWindow,public FocusLineEditKeyEvent,public AlertDialog::ButtonClickedListener
{
    Q_OBJECT

public:
    enum Status{
      reConnecting,
      ConnectFail,
      ConnectSuc
    };

    class FocusLineEdit :public QLineEdit{
    public:
        FocusLineEdit(QWidget* p):QLineEdit(p){}
        void setKeyEventListener(FocusLineEditKeyEvent* listener);
        void keyPressEvent(QKeyEvent *) override;
        void keyReleaseEvent(QKeyEvent *event) override;
        private:FocusLineEditKeyEvent* mListener;
    };
    void countDownToClosePageOnMain();
    explicit RemoteDesktopWindow(QWidget *parent = nullptr);
    void paintEvent(QPaintEvent *event) override;
    ~RemoteDesktopWindow();
    void setDragEvent(DragEvent*);
    void setDragEventOffset(QPoint* point);
    void setImageData(int w,int h,int bytePerIpxel,uint8_t* data);
    void mouseMoveEvent(QMouseEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void fKeyPressEvent(QKeyEvent *event) override;
    void fKeyReleaseEvent(QKeyEvent *event) override;

    void focusInEvent(QFocusEvent *event) override;
    void focusOutEvent(QFocusEvent *event) override;
    void wheelEvent(QWheelEvent *event) override;
    void setFocusPosition(const int x ,const int y);
//    void setId(long id);
    void setManager(RdpWindowManager * manager);
    void closeEvent(QCloseEvent *event) override;
    void resizeEvent(QResizeEvent *event) override;

    void onButtonCliecked(int index) override;
    Q_INVOKABLE
    void showDialogOnMainThread(QList<QString>,const QString& msg);
    Q_INVOKABLE
    void dismissDialogOnMainThread();
//    void setWindowClosedListener(WindowClosedListener * listener);
    void setStatus(Status type);
    Q_INVOKABLE
    void stopConnectingAnim();
    Q_INVOKABLE
    void startConnectingAnim();
    void dismissDialog();
    void showErrorDialogToReTry(const string& msg);

    void showErrorDialogToClose(const string& msg);
    bool isShowErrorDialog();
    Q_INVOKABLE
    void refresh();
    void setReconnectable(bool canReconnect){
        this->_canReConnect = canReconnect;
    }
    void setWindowTitleStd(const string& title){
        setWindowTitle(QString(title.c_str()));
    }

    void setWindowIconStd(const string& iconpath){
        setWindowIcon(QIcon(QString(iconpath.c_str())));
    }
 private:
    QPoint _cusPos;
    QTimer * _activeListener = nullptr;
    bool _hasPlateToClosePage = false;
    QImage _image;
    PlateToClosePage* _plateToClosePage = nullptr;
    bool _canReConnect = true;
    ClickedLabel* _menuBtn;
    QPoint* mDragEventOffset = nullptr;
    FocusLineEdit* mEdit;
//    WindowClosedListener* _closelistener = nullptr;
    DragEvent* mDragEvent = nullptr;
//    QPainter* _painter;
//    long id = 0;
    QThread* _uithread = nullptr;
    qreal _wscale = 0;
    qreal _hscale = 0;
    RdpWindowManager* _manager = nullptr;
    qreal _focus_x_percent = 0;
    qreal _focus_y_percent = 0;
    QTimeLine* mProgTimeLine = nullptr;
    int mConnectingProg = 0;
    AlertDialog * mAlertDialog = nullptr;
//    using ConnectType = RdpWindowManager::ConnectType;
    Status mStatus;
    Q_INVOKABLE
    void countDownToClosePage();
};


#endif // REMOTEDESKTOPWINDOW_H
