#include "settingpage.h"
#include <QPainter>
#include <QPushButton>
#include <QFileDialog>
#include "QFile"
#include "QStandardPaths"
#include "dataManager/cappsetting.h"
#include "QDir"
#include "QDateTime"
#include "utils/widgetutils.h"
#include "QLineEdit"
#include "QHBoxLayout"
#include "widgets/imageview.h"
#include "widgets/PopupWindow.h"
#include "widgets/expandlistpop.h"
#include "utils/commonlog.h"

extern CAppSetting* g_cappSetting;
SettingPage::SettingPage(QWidget *parent) : QWidget(parent)
{
    setWindowFlags(Qt::Drawer|Qt::FramelessWindowHint);
    mWindowRadiusAble = g_cappSetting->getWindowRadius() > 0;
    if(mWindowRadiusAble)setAttribute(Qt::WA_TranslucentBackground);
    setFixedSize(mWidth,300);
    mFont.setPixelSize(15);

    QWidget* selectSDiskWidget = new QWidget(this);
    selectSDiskWidget->setGeometry(30,mDrawStartY+22,width() - 50,40);
    QHBoxLayout* selectSDiskLay = new QHBoxLayout(selectSDiskWidget);
    selectSDiskLay->setContentsMargins(0,0,0,0);
    selectSDiskLay->setSpacing(5);

    mEdit = new QLineEdit(this);
    mEdit->setText(QString( g_cappSetting->getSDiskPath().c_str()));
    mEdit->setReadOnly(true);
    mEdit->setFixedHeight(40);
//    edit->setGeometry(30,mDrawStartY+22,width() - 60,40);
    mEdit->setStyleSheet("QLineEdit{padding-left:5px;padding-right:5px;border:1px solid #888888;border-radius:6px;background:#fafafa}");
    selectSDiskLay->addWidget(mEdit,1);

    QPushButton* selectSDisk = new QPushButton(this);
    selectSDisk->setFlat(true);
    selectSDisk->setText("选择");
    selectSDisk->setStyleSheet("QPushButton{color:#333333;padding: 5px 5px}");
    selectSDiskLay->addWidget(selectSDisk);
    connect(selectSDisk,&QPushButton::clicked,this,[=]{
        KT_Debug("g_appSetting.getSDiskPath() %s" , g_cappSetting->getSDiskPath().c_str());
        QString dirPath = QFileDialog::getExistingDirectory(parent,"选择S盘路径",QString(g_cappSetting->getSDiskPath().c_str()));
        if(dirPath.isNull() || dirPath.isEmpty())return;

        KT_Debug(dirPath.toStdString().c_str());
        //--------平台不同，使用文件读写强制获取该路径的文件权限--------
        bool isValid = true;
        QDir sDir(dirPath);
        if(!sDir.exists()){//支持手动编辑路径
            isValid = sDir.mkpath(dirPath);
        }else{
            QDateTime t;
            QFile tmp(dirPath+"/.temp3xr");
            if(tmp.exists()){
                isValid = tmp.remove();
            }else{
                if(!tmp.open(QIODevice::ReadWrite|QIODevice::Text)){//获取读写权限
                   isValid = false;
                }
                tmp.close();
                if(isValid){
                    tmp.remove();
                }
            }
        }

        if(isValid){
            mEdit->setText(dirPath);
            mToast->showMesssage("设置成功,请点击保存!");
        }else{
            mToast->showMesssage("设置失败，您没有权限访问该路径");
        }
    });

    mSelect = new QPushButton(this);
    mSelect->setFlat(true);

    int time = g_cappSetting->getBrokenTime();
    int loopIndex = 0;
    for(int t : _times){
        if(t == time){
            mSelectTimeIndex = loopIndex;
        }
        loopIndex ++;
    }

    mSelect->setText(_timesNames[mSelectTimeIndex]);
    mSelect->setFixedHeight(40);
    mSelect->setGeometry(30,mDrawStartY+131,width() - 60,40);
    mSelect->setStyleSheet("QWidget{text-align:left;padding-left:5px;padding-right:5px;border:1px solid #888888;border-radius:6px;background:#fafafa}");

    ImageView * accExpBtn = new ImageView(mSelect);
    accExpBtn->setFixedSize(35,35);
    accExpBtn->setRadius(100);
    accExpBtn->waveCenter(true);
    accExpBtn->setPadding(7,7,7,7);
    accExpBtn->setSvgPixMapSrc(":/images/expand_more.svg",35,35);
    accExpBtn->setGeometry(width()- 100,3,35,35);

    connect(accExpBtn,&ImageView::clicked,this,[=]{
        QPoint gp = mSelect->mapToGlobal(QPoint());
        showTimeMsgWin(mSelect->width()+20,gp.x()-10,gp.y()+mSelect->height());
    });

    connect(mSelect,&QPushButton::clicked,this,[=]{
        QPoint gp = mSelect->mapToGlobal(QPoint());
        showTimeMsgWin(mSelect->width()+20,gp.x()-10,gp.y()+mSelect->height());
    });
    QFont font;
    font.setPixelSize(15);

    int padding = 30;
    int margin = 20;
    int y = 240;
    int btnHeight = 30;
    int btnWidth = (width() - margin -padding* 2) / 2;

    QPushButton* cancelBtn = new QPushButton(this);
    cancelBtn->setFont(font);
    cancelBtn->setFixedSize(btnWidth,btnHeight);
    cancelBtn->setGeometry(padding,y,cancelBtn->width(),cancelBtn->height());
    cancelBtn->setText("取消");
    connect(cancelBtn,&QPushButton::clicked,this,[=]{
        close();
    });
    cancelBtn->setStyleSheet("QPushButton{border:none;border-radius:4px;padding:5px 6px;background-color:#cc74caf7;color:#ffffff}"
                             "QPushButton:hover{background-color:rgb(116,202,247);}");

    QPushButton* sureBtn = new QPushButton(this);
    sureBtn->setFont(font);
    sureBtn->setFixedSize(btnWidth,btnHeight);
    sureBtn->setGeometry(width() - padding - btnWidth,y,cancelBtn->width(),cancelBtn->height());
    sureBtn->setText("保存");
    connect(sureBtn,&QPushButton::clicked,this,[=]{
        QString dir = mEdit->text();
        string dirStd = dir.toStdString();
        g_cappSetting->setSDiskPath(dirStd);
        g_cappSetting->setBrokenTime(_times[mSelectTimeIndex]);
        close();
    });
    sureBtn->setStyleSheet("QPushButton{border:none;border-radius:4px;padding:5px 6px;background-color:#cc74caf7;color:#ffffff}"
                             "QPushButton:hover{background-color:rgb(116,202,247);}");

    mToast = new ToastWidget(this);
}

void SettingPage::showTimeMsgWin(int width,int showX,int showY){
    ExpandListPop* p = new ExpandListPop(width);
    p->setItemPadding(15,0,15,0);

    p->setItems(_timesNames);
    p->showPop(showX,showY);

    connect(p,&ExpandListPop::itemClick,this,[=](QString& s, int index){
        mSelect->setText(s);
        mSelectTimeIndex = index;
        p->dismiss();
    });
}

void SettingPage::closeEvent(QCloseEvent *){
    emit closed();
    deleteLater();
}

void SettingPage::show(int x,int y){
    move(x,y);
    QWidget::show();
}

void SettingPage::paintEvent(QPaintEvent *){
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    QPen pen;
    pen.setWidth(1);
    pen.setColor(QColor(188,188,188));
    painter.setPen(pen);
    QColor bgColor = QColor(0xfa,0xfa,0xfa);
    painter.setBrush(bgColor);
    int radius = mWindowRadiusAble ? 9 : 0;
    painter.drawRoundedRect(rect(),radius,radius);

    pen.setColor(QColor(0x77,0x77,0x77));
    pen.setWidthF(1.0/2.44);
    painter.setPen(pen);
    painter.setFont(mFont);

    int rectHeight = 75;
    painter.drawRoundedRect(QRect(15,mDrawStartY,width()-30,rectHeight),6,6);
    painter.setBrush(bgColor);
    QRect rect = WidgetUtils::getTextRect(mFont,"虚拟磁盘(S:)");
    mLabelHeight = rect.height();
    painter.setPen(Qt::NoPen);
    painter.drawRect(29,mDrawStartY-mLabelHeight,rect.width()+3,mDrawStartY);
    pen.setColor(QColor(0x33,0x33,0x33));
    painter.setPen(pen);
    painter.drawText(30,mDrawStartY+mLabelHeight/4,"虚拟磁盘(S:)");

    int top = rectHeight + mDrawStartY + 33;
    painter.drawRoundedRect(QRect(15,top,width()-30,rectHeight),6,6);
    painter.setBrush(bgColor);

    painter.setPen(Qt::NoPen);
    rect = WidgetUtils::getTextRect(mFont,"空闲超时");
    painter.drawRect(29,top-mLabelHeight,rect.width()+3,mDrawStartY);
    pen.setColor(QColor(0x33,0x33,0x33));
    painter.setPen(pen);
    painter.drawText(30,top+mLabelHeight/4,"空闲超时");
}
