#include "stringutils.h"
#include "QUuid"
#include "QCryptographicHash"
#include "QDebug"
#include "qaesencryption.h"
#include "iconv.h"
StringUtils::StringUtils()
{

}

ushort StringUtils::unicodeChar(char c){
    return QChar(c).unicode();
}

QString StringUtils::md5(const QString& dataStr){
    QByteArray data = QCryptographicHash::hash ( dataStr.toLatin1(), QCryptographicHash::Md5 );
    return QString(data.toHex());
}

QString StringUtils::getUUid(){
    QUuid uuid = QUuid::createUuid();
    QString uuidStr = uuid.toString();
    uuidStr.remove(0,1);
    uuidStr.remove(uuidStr.count() - 1 , 1);
    uuidStr.remove(QRegExp("-"));
    return QString(uuidStr);
}

QString StringUtils::aesCBCPKCS5PADDINGEncode(const QString& pcInput,const QString& key, const QString& iv){
    return aesCBCPKCS5PADDINGEncode2Byte(pcInput,key,iv);
}

QByteArray StringUtils::aesCBCPKCS5PADDINGEncode2Byte(const QString& pcInput,const QString& key, const QString& iv){
    QAESEncryption encryption(QAESEncryption::AES_128, QAESEncryption::CBC,QAESEncryption::PKCS7);
    QByteArray encodedText = encryption.encode(pcInput.toLocal8Bit(),key.toLocal8Bit(),iv.toLocal8Bit());
    return encodedText.toBase64();
}

QByteArray StringUtils::aesCBCPKCS5PADDINGDecode(const QString& pcInput,const QString& key, const QString& iv){
    QAESEncryption encryption(QAESEncryption::AES_128, QAESEncryption::CBC,QAESEncryption::PKCS7);
    QByteArray dataDecode = QByteArray::fromBase64(pcInput.toLocal8Bit());

    QByteArray decodeData = encryption.decode(dataDecode,key.toLocal8Bit(),iv.toLocal8Bit());
    int size = decodeData.length();
    int removeNum = (int)decodeData[size - 1];
    return decodeData.remove(size - removeNum , removeNum);
}

QString StringUtils::base64QString(const QString& encrypt_data){
    return QString(base64((unsigned char*)encrypt_data.toLocal8Bit().data(),encrypt_data.length()).c_str());
}
static const std::string base64_chars =
"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
"abcdefghijklmnopqrstuvwxyz"
"0123456789+/";
std::string StringUtils::base64(unsigned char* encrypt_data,int size)
{
    std::string ret;
        int i = 0;
        int j = 0;
        unsigned char char_array_3[3];
        unsigned char char_array_4[4];

        while (size--)
        {
            char_array_3[i++] = *(encrypt_data++);
            if(i == 3)
            {
                char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
                char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
                char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
                char_array_4[3] = char_array_3[2] & 0x3f;
                for(i = 0; (i <4) ; i++)
                {
                    ret += base64_chars[char_array_4[i]];
                }
                i = 0;
            }
        }
        if(i)
        {
            for(j = i; j < 3; j++)
            {
                char_array_3[j] = '\0';
            }

            char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
            char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
            char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
            char_array_4[3] = char_array_3[2] & 0x3f;

            for(j = 0; (j < i + 1); j++)
            {
                ret += base64_chars[char_array_4[j]];
            }

            while((i++ < 3))
            {
                ret += '=';
            }

        }
        return ret;

}

bool StringUtils::is_base64(const char c){
    return (isalnum(c) || (c == '+') || (c == '/'));
}

bool StringUtils::gbk2utf8(char* src, char* dest, size_t inlen)
{
    char *inbuf = src;
    size_t outlen = inlen * 4;
    char *outbuf = dest;
//    memset(outbuf, 0, outlen);
    iconv_t cd = iconv_open("UTF-8", "GBK");
    /* iconv接口会改变inbuf和outbuf，所以不能直接使用src和dest */
    size_t res = iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
   iconv_close(cd);
    return (0 == res) ? true : false;
}

QString StringUtils::gbk2Utf8String(char* src,size_t inlen){
    const int len = inlen;
    char outbuf[len];
    size_t z = inlen;
    gbk2utf8(src,outbuf,z);
    return QString("%1").arg(outbuf);
}

std::string StringUtils::base64_decode(std::string const & encoded_string)
{
    int in_len = (int) encoded_string.size();
    int i = 0;
    int j = 0;
    int in_ = 0;
    unsigned char char_array_4[4], char_array_3[3];
    std::string ret;

    while (in_len-- && ( encoded_string[in_] != '=') && is_base64(encoded_string[in_])) {
        char_array_4[i++] = encoded_string[in_]; in_++;
        if (i ==4) {
            for (i = 0; i <4; i++)
                char_array_4[i] = base64_chars.find(char_array_4[i]);

            char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
            char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
            char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

            for (i = 0; (i < 3); i++)
                ret += char_array_3[i];
            i = 0;
        }
    }
    if (i) {
        for (j = i; j <4; j++)
            char_array_4[j] = 0;

        for (j = 0; j <4; j++)
            char_array_4[j] = base64_chars.find(char_array_4[j]);

        char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
        char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
        char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

        for (j = 0; (j < i - 1); j++) ret += char_array_3[j];
    }

    return ret;
}

