#include "PopupWindow.h"
#include "QPainter"
#include "QGraphicsDropShadowEffect"
#include "QMouseEvent"
#include "QTimeLine"
#include "QLabel"
#include "QDebug"
#include "QTimer"
#include "dataManager/cappsetting.h"
#include <QThread>
using namespace ::std;
extern CAppSetting* g_cappSetting;
PopupMenuWindow::PopupMenuWindow(QWidget *parent,Qt::WindowFlags flags):QWidget(parent){
    QGraphicsDropShadowEffect *shadow_effect = new QGraphicsDropShadowEffect(this);
    shadow_effect->setOffset(0, 0);
    _radiusAble = g_cappSetting->getWindowRadius() > 0;
    int shadowAlfa = 180;
    #if defined(Q_OS_MACOS)
        shadowAlfa = 100;
    #endif
    shadow_effect->setColor(QColor(0,0,0,shadowAlfa));
    shadow_effect->setBlurRadius(_radius);
    setGraphicsEffect(shadow_effect);
    if(parent == nullptr){
        setWindowFlags(flags|Qt::Window);
        setAttribute(Qt::WA_TranslucentBackground);
        if(!_radiusAble){
            _padding = 0;
            _radius = 0;
        }
    }else{
        _padding = 0 ;
    }

    setMouseTracking(true);

    _moreBip = QPixmap::fromImage(QImage(":/images/arr_right.png")).scaledToWidth(12,Qt::SmoothTransformation);

 }

void PopupMenuWindow::paintEvent(QPaintEvent *) {
    const int w = width();

    if(w < _truewidth){
        setFixedWidth(_truewidth);
        return;
    }
    _truewidth = w;

    const int left = _padding+_itemLeftPadding;

    QPainter painter(this);
    painter.setPen(Qt::NoPen);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);

    painter.setBrush(_bgColor);
    painter.drawRoundedRect(_padding,_padding,width()-_padding*2,height() - _padding*2,_radius,_radius);

//        painter.setBrush(Qt::blue);
//        painter.drawRect(_padding,_padding,this->width(),30);

    QPen pen;
    pen.setWidthF(1.0 / 2.54);
    painter.setBrush(Qt::NoBrush);
    pen.setColor(_labelColor);
    painter.setPen(pen);
    QFont font;
    font.setPixelSize(14);
    font.setWeight(0);
    painter.setFont(font);

    int startTop = _padding;
    int lineEnd = w- _itemLeftPadding;

    int itemsSize = items.size();
    for(int index = 0; index < itemsSize ; index ++){
        QString data = items[index];
        if(data.isEmpty()){
            painter.drawLine(_itemLeftPadding,startTop,lineEnd,startTop);
        }else{
            QColor selectColor = _labelColor;
            if(_mouseEntered && _focusIndex == index){
                selectColor = Qt::white;
                drawFocusRect(startTop);
                pen.setColor(selectColor);
                painter.setPen(pen);
            }

            if(selectStatus[index])drawSelect(_padding+12,startTop+(_itemHeight - _SelectRecH + 1) / 2,selectColor);
            if(std::find(_existMoreItem.begin(),_existMoreItem.end(),index)!=_existMoreItem.end())painter.drawPixmap(width() - _itemLeftPadding - _moreBip.width(),(_itemHeight-_moreBip.height())/2+startTop+1,_moreBip);
            painter.drawText(left,startTop,w,_itemHeight,Qt::AlignVCenter,data);
            startTop += _itemHeight;

            pen.setColor(_labelColor);
            painter.setPen(pen);
        }
    }
    if(!_radiusAble && parentWidget() == nullptr){
        QPen pen;
        pen.setWidth(1);
        pen.setColor(QColor(0xaa,0xaa,0xaa));
        painter.setPen(pen);
        painter.drawRect(rect());
    }
 }

void PopupMenuWindow::drawFocusRect(int startTop){
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(Qt::NoPen);
    painter.setBrush(_focusColor);
    int padding = 5;
    painter.drawRoundedRect(_padding + padding,startTop + padding , width() - padding * 2 - _padding * 2, _itemHeight - padding * 2 , 5,5);
}

void PopupMenuWindow::drawSelect(int left ,int top,QColor color){
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(color);
    QRect rect(left,top,_selectRecW,_SelectRecH);
    int endX = rect.left()+rect.width()*0.26;
    int endY = rect.top()+rect.height() * 0.8;
    painter.drawLine(rect.left(),rect.top()+rect.height() * 0.4,endX,endY);
    painter.drawLine(endX,endY,rect.right(),rect.top());
}

void PopupMenuWindow::addItem(const QString& label,bool checked,bool existMore){
    if(label.isEmpty())return;
    selectStatus.push_back(checked);

    items.push_back(label);
    QFont font;
    font.setPixelSize(14);
    QFontMetrics m(font);
    if(existMore){
        _existMoreItem.emplace_back(items.size() - 1);
    }
#if (QT_VERSION <= QT_VERSION_CHECK(5,14,0))
    int w = m.width(label);
#else
    int w = m.horizontalAdvance(label);
#endif
    if(w > _item_max_width){
        _item_max_width = w;
        const int left = _padding+_itemLeftPadding;
        _truewidth = _item_max_width + left * 2 + 1+ (_existMoreItem.size() > 0 ? (6 + _moreBip.width()) : 0);
    }
}


void PopupMenuWindow::mouseMoveEvent(QMouseEvent *e) {
    e->setAccepted(true);
    if(!_mouseEntered)return;
    int y = e->y();
    int size = _heightSign.size();
    int fIndex = -1;
    for(int i=0;i< size ;i ++){
        int bottom = _heightSign[i];
        if(y > bottom - _itemHeight && y < bottom){
            fIndex = i;
            break;
        }
    }

    if(fIndex < 0 || fIndex == _focusIndex)return;
    _focusIndex = fIndex;
    emit itemFocused(_focusIndex);
    if(mItemFocusListener)mItemFocusListener(_focusIndex);
    update();
}

void PopupMenuWindow::mouseReleaseEvent(QMouseEvent * e){
    e->setAccepted(true);
    if(!_mouseEntered){
        dismiss();

    }else{
        if(_listener != nullptr){
            _listener->popSelected(this,_focusIndex);
        }
        emit itemClick(_focusIndex);
        if(mItemClickListener){
            mItemClickListener(_focusIndex);
        }
    }
 }


