#ifndef POPUPWINDOW_H
#define POPUPWINDOW_H
#include "QWidget"
#include <QMap>
#include <QDebug>
#include <QTimer>
#include <string>
#include "utils/mainlooper.h"
using namespace std;

class PopupMenuWindow :public QWidget{
    Q_OBJECT
public:
    class SelectPopItemListener{
    public:
        virtual void popSelected(PopupMenuWindow*,int) = 0;
    };
    explicit PopupMenuWindow(QWidget *parent = Q_NULLPTR,Qt::WindowFlags flags = Qt::Popup);
    ~PopupMenuWindow(){
        setParent(nullptr);

    }

    void setSigin(const QString& sign){
        this->_sign = sign;
    }

    QString& getSign(){
        return this->_sign;
    }

    int getItemHeight(){
        return _itemHeight;
    }

    int getTrueWidth(){
        return _truewidth;
    }

    int getOutPadding(){
        return _padding;
    }

    void setSelectItemLisenter(SelectPopItemListener* lister){
        this->_listener = lister;
    }

    void addLine(){
        items.push_back("");
        selectStatus.push_back(false);
    }

    int measureHeight(){
       int height = 0;
       _heightSign.clear();
       for(QString d : items){
           height += d.isEmpty() ? 1 : (_itemHeight);
           _heightSign.push_back(height);
       }
       return height;
    }

    void showPopup(int x,int y){
        setFixedHeight(measureHeight()+_padding*2);
        qDebug() << "x,y" << x << y << "," << x-_padding << y - _padding;
        move(x-_padding ,y - _padding);
        show();
     }

    void enterEvent(QEvent *) override{
        _mouseEntered = true;
    }

    void leaveEvent(QEvent *) override{
        _mouseEntered = false;
        update();
        _focusIndex = -1;
    }
    void closeEvent(QCloseEvent *) override{
        emit closed();
        if(mCloseListener)mCloseListener();
        destroy();
    }

    void destroy(){
        if(!_destroyed){
            setParent(nullptr);
            this->deleteLater();
        }
        _destroyed = true;
    }

    void dismiss(){
        this->close();
        destroy();
    }

    void paintEvent(QPaintEvent *) override;
    void drawFocusRect(int startTop);

    void drawSelect(int left ,int top,QColor color);

    void addItemSTD(const string& label,bool checked = false,bool existMore=false){
        addItem(QString(label.c_str()),checked,existMore);
    }

    void addItem(const QString& label,bool checked = false,bool existMore=false);
    void mouseMoveEvent(QMouseEvent *e) override;

    void focusInEvent(QFocusEvent *event) override{
        qDebug() << "focusInEvent";
    }

    void focusOutEvent(QFocusEvent *event) override{
        qDebug() << "focusOutEvent";
    }

    void mousePressEvent(QMouseEvent *event) override{
        qDebug() << "mousePressEvent";
    }

    void mouseReleaseEvent(QMouseEvent * e) override;

    int getItemCount(){
        return items.size();
    }

    void setCloseListener(const std::function<void ()>& listener){
        this->mCloseListener = listener;
    }

    void setItemClickListener(const std::function<void (int)> &listener){
        this->mItemClickListener = listener;
    }

    void setItemFocusListener(const std::function<void (int)>& listener){
        this->mItemFocusListener = listener;
    }

signals:
    void itemClick(int itemIndex);
    void itemFocused(int itemIndex);
    void closed();
private:
    bool _radiusAble = true;
    std::vector<QString> items;//存储标记的Label，“”表示是直线
    std::vector<bool> selectStatus; //存储被选中的item
    std::vector<int> _heightSign;//存储每个item的底部位置，用于focus绘制，减少计算压力
    std::function<void ()> mCloseListener = nullptr;
    std::function<void (int)> mItemClickListener = nullptr;
    std::function<void (int)> mItemFocusListener = nullptr;
    int _radius = 7;
    std::vector<int> _existMoreItem;
    QPixmap _moreBip;
    int _item_max_width = 0;
    int _padding = _radius;
    int _truewidth = 0;

    SelectPopItemListener* _listener = nullptr;
    bool _mouseEntered = false;
    QColor _bgColor = QColor(250,250,250,233);
    QColor _labelColor = QColor(90,90,90);
    int _itemHeight = 36;
    int _itemLeftPadding = 30;
    int _selectRecW = 11;
    int _SelectRecH = 9;
    int _focusIndex = -1;
    QString _sign;

    bool _destroyed = false;
    const QColor _focusColor = QColor(126,200,246);
};
#endif // POPUPWINDOW_H
