#include "adjustlengthpushbutton.h"
#include "QResizeEvent"
#include "utils/widgetutils.h"
#include "QPainter"
#include "QMouseEvent"

AdjustLengthPushButton::AdjustLengthPushButton(QWidget* p):ClickedLabel(p){
    setMouseTracking(true);
    mNavLabel = new AdjustLengthPushButtonTip;
    mNavLabel->setWindowFlags(Qt::ToolTip |Qt::FramelessWindowHint);
    mNavLabel->setAttribute(Qt::WA_TranslucentBackground);
    mNavLabel->setAlignment(Qt::AlignCenter);
    mNavLabel->setStyleSheet("QLabel{color:#cc444444}");
}

void AdjustLengthPushButton::mouseMoveEvent(QMouseEvent *e){
    if(mNeedShowNav && mNavLabel->isVisible()){
        mNavLabel->move(e->globalX()+5,e->globalY()+5);
        mNavLabel->raise();
//        mNavLabel->show();
    }
}

void AdjustLengthPushButtonTip::paintEvent(QPaintEvent * e){
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    QPen pen;
    pen.setColor(QColor(150,150,150));
    pen.setWidthF(0.5);
    painter.setPen(pen);
    painter.setBrush(QColor(246,246,246,220));
    painter.drawRoundedRect(1,1,width()-2,height()-2,6,6);
    QLabel::paintEvent(e);
}

void AdjustLengthPushButton::enterEvent(QEvent *){
    if(mNeedShowNav)mNavLabel->show();
}

void AdjustLengthPushButton::leaveEvent(QEvent *){
    if(mNeedShowNav)mNavLabel->hide();
}

void AdjustLengthPushButton::paintEvent(QPaintEvent * e){
    ClickedLabel::paintEvent(e);
//    QPainter painter(this);
//    painter.setBrush(Qt::red);
//    painter.drawRect(rect());
}

void AdjustLengthPushButton::resizeEvent(QResizeEvent *){
    QFont f = font();

    int w = WidgetUtils::getTextRect(f,mUserText).width();
    setText(mUserText,false);

    QString suf = "..";
    int minW = WidgetUtils::getTextRect(f,suf).width();
    int vw = width() - mPaddingLeft - mPaddingRight;
    mNeedShowNav = false;
    QString label = QString(text());
    if(vw >minW && w > vw){
        mNeedShowNav = true;
        while(w > vw){
            label.remove(label.size()-1,1);
            target = QString(label);
            target.append(suf);
            w = WidgetUtils::getTextRect(f,target).width();
        }
        setText(target,false);
    }
}

QString AdjustLengthPushButton::userText(){
    return mUserText;
}

void AdjustLengthPushButton::setText(const QString &text,bool fromUser){
    QLabel::setText(text);

    if(fromUser){
        mUserText = text;
        QFont f = font();
        mTextWidth = WidgetUtils::getTextWidth(f,mUserText);
        mNavLabel->setContentsMargins(8,3,8,3);

        mNavLabel->setWordWrap(true);
        mNavLabel->setFixedWidth(mTextWidth+20);
//        mNavLabel->setMaximumWidth(500);
        mNavLabel->setText(mUserText);
        resizeEvent(nullptr);
    }
}
