#include "alertdialog.h"
#include "QPainter"
#include "QVBoxLayout"
#include "QPalette"
#include "QPushButton"

AlertDialog::AlertDialog(QWidget* parent,int paddingLeft,int paddingTop,int paddingRight,int paddingBottom): QWidget(parent)
{
    Q_ASSERT(parent != nullptr);
    mPaddingLeft = paddingLeft;
    mPaddingTop = paddingTop;
    mPaddingRight = paddingRight;
    mPaddingBottom = paddingBottom;
    QVBoxLayout* lay = new QVBoxLayout(parent);
    parent->setLayout(lay);

    QVBoxLayout * thisLay = new QVBoxLayout();
    setLayout(thisLay);
    thisLay->setAlignment(Qt::AlignCenter);

    QWidget* dialogContent = new QWidget();

    QVBoxLayout* layout = new QVBoxLayout(dialogContent);
    int lrPadding = 20;
    layout->setContentsMargins(lrPadding,10,lrPadding,12);
//    layout->setAlignment(Qt::AlignHCenter);
    int width = 300;
    dialogContent->setFixedWidth(width);

    mTitleLabel = new QLabel;
    mTitleLabel->setAlignment(Qt::AlignCenter);
    QFont font;
    font.setPixelSize(16);
    mTitleLabel->setFont(font);
    layout->addWidget(mTitleLabel);

    QFont cfont;
    cfont.setPixelSize(14);

    mContentLabel = new QLabel;
    mContentLabel->setFixedWidth(width - lrPadding * 2);
    mContentLabel->setFont(cfont);
    mContentLabel->setWordWrap(true);
    mContentLabel->setText("通过CSSborder-radius属性,可以实现任何元素的“圆角”样式通过CSSborder-radius属性,可以实现任何元素的“圆角”样式通过CSSborder-radius属性,可以实现任何元素的“圆角”样式通过CSSborder-radius属性,可以实现任何元素的“圆角”样式");
    layout->addWidget(mContentLabel);
    layout->addSpacing(5);

    mButtonsLay = new QHBoxLayout;
    mButtonsLay->setContentsMargins(0,0,0,0);
    mButtonsLay->setAlignment(Qt::AlignRight);
    layout->addLayout(mButtonsLay);
//    layout->setSizeConstraint(QLayout::SetFixedSize);

    QPalette palette;
    dialogContent->setAutoFillBackground(true);
    palette.setColor(QPalette::Window,Qt::blue);
    dialogContent->setPalette(palette);
    dialogContent->setStyleSheet("QWidget{border-radius: 10px;background: #ffffff;}");

    thisLay->setContentsMargins(paddingLeft,paddingTop,paddingRight,paddingBottom);
    thisLay->addWidget(dialogContent);

    lay->setContentsMargins(0,0,0,0);
    lay->addWidget(this);
    hide();
}

void AlertDialog::paintEvent(QPaintEvent *){
    QPainter painter(this);
    painter.setBrush(QColor(0,0,0,mBgColorAlpha));
    painter.setPen(Qt::NoPen);
    painter.drawRoundedRect(rect(),mRadius,mRadius);
}

void AlertDialog::mousePressEvent(QMouseEvent *){

}

void AlertDialog::mouseReleaseEvent(QMouseEvent *){

}

void AlertDialog::setTitle(const QString& title){
    this->mTitleLabel->setText(title);
}

void AlertDialog::setMessage(const QString& message){
    this->mContentLabel->setText(message);
}

void AlertDialog::setButtons(std::vector<QString>& buttons){
    QLayoutItem *child;
     while ((child = mButtonsLay->takeAt(0)) != 0){
        if(child->widget()){
            child->widget()->setParent(NULL);
        }
        delete child;
     }

    int index = 0;
    int size = buttons.size();
    for(QString& btnText : buttons){
        QPushButton* btn = new QPushButton;
        QFont font;
        font.setPixelSize(13);
        btn->setFont(font);
        connect(btn,&QPushButton::clicked,this,[=](bool){
            if(mListener)mListener->onButtonCliecked(index);
            emit buttonClick(index);
            if(mClickListener)mClickListener(index);
            std::vector<QString> btn = {"取消","确定"};
            setButtons(btn);
        });

        btn->setText(btnText);
        mButtonsLay->addWidget(btn);
        if(index != size - 1)mButtonsLay->addSpacing(15);
        index ++;
    }
}

void AlertDialog::setOnButtonClickListener(ButtonClickedListener * listener){
    this->mListener = listener;
}

void AlertDialog::show(){
    mShowAnimTimerLine = new QTimeLine(220,this);
    mShowAnimTimerLine->setFrameRange(0,30);
    connect(mShowAnimTimerLine,&QTimeLine::frameChanged,[=](int frame){
        mBgColorAlpha = frame;
        update();
    });
    mShowAnimTimerLine->start();
    raise();
    QWidget::show();
}

void AlertDialog::dismiss(){
    if(mShowAnimTimerLine){
        mShowAnimTimerLine->stop();
        mShowAnimTimerLine = nullptr;
    }
    hide();
}

