#ifndef ALERTDIALOG_H
#define ALERTDIALOG_H
#include "QWidget"
#include "QLabel"
#include "QHBoxLayout"
#include "QTimeLine"
using namespace std;
#include <string>

class AlertDialog:public QWidget
{
    Q_OBJECT
public:
    class ButtonClickedListener{
       public: virtual void onButtonCliecked(int index) = 0;
    };
private:
    QLabel* mTitleLabel;
    QLabel* mContentLabel;
    std::function<void (int)> mClickListener = nullptr;
    int mRadius = 0;
    int mPaddingLeft=0,mPaddingRight =0,mPaddingTop =0,mPaddingBottom=0;
    QHBoxLayout* mButtonsLay;
    int mBgColorAlpha = 0;
    ButtonClickedListener * mListener = nullptr;
    QTimeLine* mShowAnimTimerLine = nullptr;
public:
    void setOnButtonClickListener(ButtonClickedListener * listener);
    explicit AlertDialog(QWidget* parent = nullptr,int paddingLeft=0,int paddingTop=0,int paddingRight=0,int paddingBottom=0);
    void paintEvent(QPaintEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void show();
    void setRadius(int radius){
        mRadius = radius;
    }
    void setButtonsStd(std::vector<string>& buttons){
        std::vector<QString> labels;
        for(string& lable : buttons){
            labels.push_back(QString(lable.c_str()));
        }
        setButtons(labels);
    }
    void dismiss();
    void setTitle(const QString& title);
    void setMessage(const QString& message);

    void setTitleStd(const string& title){
        setTitle(QString(title.c_str()));
    }
    void setMessageStd(const string& message){
        setMessage(QString(message.c_str()));
    }

    void setButtons(std::vector<QString>& buttons);
    void listenerButtonClick(std::function<void (int)> listener){
        this->mClickListener = listener;
    }
signals:
    void buttonClick(int index);
};

#endif // ALERTDIALOG_H
