#include "basewindow.h"
#include "QPainter"
#include "QApplication"
#include "QScreen"
#include "widgets/windowclosebtn.cpp"
#include "widgets/windowminbtn.cpp"
#include "widgets/windowresizebtn.h"
#include "QHBoxLayout"
#include "QPalette"
#include "dragmovedwidget.h"
#include "QLabel"

BaseWindow::BaseWindow(QWidget* parent):QWidget(parent)
  ,mBordColor(QColor(100,100,100)),mMovedPoint(new QPoint)
{
    setWindowFlags(Qt::Window |Qt::FramelessWindowHint);
    setAttribute(Qt::WA_TranslucentBackground);
    setMouseTracking(true);
    mViewLay = new QVBoxLayout(this);
    mViewLay->setContentsMargins(0,0,0,0);
    mViewLay->setSpacing(0);

    QSize size(BUTTON_SIZE,BUTTON_SIZE);

    WindowCloseButton* c = new WindowCloseButton();
    c->setHost(this);
    c->setFixedSize(size);

    mRzBtn = new WindowResizeBtn();
    mRzBtn->setHost(this);
    mRzBtn->setFixedSize(size);

    WindowMinButton* minBtn = new WindowMinButton;
    minBtn->setHost(this);
    minBtn->setFixedSize(size);

    DragMovedWidget* titleWidgt = new DragMovedWidget;
    titleWidgt->setDragEvent(this);

    titleWidgt->setFixedHeight(TITILEBAR_HEIGHT);
    mTitleLay = new QHBoxLayout(titleWidgt);
    mTitleLay->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    mTitleLay->setContentsMargins(10,0,10,0);
    mTitleLay->setSpacing(16);

    mTitleContentLay = new QHBoxLayout;
    mTitleContentLay->setContentsMargins(0,0,0,0);
    mTitleContentLay->setSpacing(16);
    mTitleLay->addLayout(mTitleContentLay);

    mTitleLay->addWidget(minBtn);
    mTitleLay->addWidget(mRzBtn);
    mTitleLay->addWidget(c);

    titleWidgt->setHost(this);

    mViewLay->addWidget(titleWidgt);
    mContainerLay = new QVBoxLayout;
    mContainerLay->setContentsMargins(1,0,1,1);
    mContainerLay->setSpacing(0);
    mViewLay->addLayout(mContainerLay);
}

int BaseWindow::buttonSize(){
    return BUTTON_SIZE;
}

void BaseWindow::mousePressEvent(QMouseEvent *e){
    changeSizeDir(e->x(),e->y());
    if(mChangeSizeDir > -1){
        mIsChangeSizeMode = true;
        mWhenDownWindowSize = QSize(this->size());
//        mWhenDownPoint = QPoint(e->pos());
        mMouseDownWindowGlobalPosition = QPoint(pos());
    }
    mMouseDownPoint.setX(e->pos().x());
    mMouseDownPoint.setY(e->pos().y());
    mDownXPercent = e->pos().x() * 1.0 / width();
    mMovedPoint->setY(mMouseDownPoint.y());
}

QHBoxLayout* BaseWindow::getTitleLay(){
    return mTitleLay;
}

QHBoxLayout* BaseWindow::getTitleContentLay(){
    return mTitleContentLay;
}

void BaseWindow::mouseReleaseEvent(QMouseEvent *){
    mIsChangeSizeMode = false;
    mMouseDownPoint.setX(-1);
    mMouseDownPoint.setY(-1);
}

int BaseWindow::titleHeight(){
    return TITILEBAR_HEIGHT;
}

void BaseWindow::mouseMoveEvent(QMouseEvent *e){
    if(mIsChangeSizeMode){
        QPoint dis = e->globalPos() - mMouseDownWindowGlobalPosition;
        switch (mChangeSizeDir) {
        case 0:
            setGeometry(mMouseDownWindowGlobalPosition.x() + dis.x(),this->y(),mWhenDownWindowSize.width() - dis.x(),height());
            break;
        case 1:
            setGeometry(mMouseDownWindowGlobalPosition.x() + dis.x(),mMouseDownWindowGlobalPosition.y()+dis.y(),mWhenDownWindowSize.width() - dis.x(),mWhenDownWindowSize.height() - dis.y());
            break;
        case 2:
            setGeometry(mMouseDownWindowGlobalPosition.x(),mMouseDownWindowGlobalPosition.y()+dis.y(),mWhenDownWindowSize.width(),mWhenDownWindowSize.height() - dis.y());
            break;
        case 3:
            setGeometry(mMouseDownWindowGlobalPosition.x(),mMouseDownWindowGlobalPosition.y()+dis.y(),dis.x(),mWhenDownWindowSize.height() - dis.y());
            break;
        case 4:
            setGeometry(mMouseDownWindowGlobalPosition.x(),mMouseDownWindowGlobalPosition.y(), dis.x(),height());
            break;
        case 5:
            setGeometry(mMouseDownWindowGlobalPosition.x(),mMouseDownWindowGlobalPosition.y(), dis.x(), dis.y());
            break;
        case 6:
            setGeometry(mMouseDownWindowGlobalPosition.x(),mMouseDownWindowGlobalPosition.y(), width(), dis.y());
            break;
        case 7:
            setGeometry(mMouseDownWindowGlobalPosition.x()+dis.x(),mMouseDownWindowGlobalPosition.y(), mWhenDownWindowSize.width()-dis.x(),dis.y());
            break;
        }
        return;
    }

    if(mIsMaximized && mMouseDownPoint.y() < TITILEBAR_HEIGHT && mMouseDownPoint.y() > 0){
        QPoint dis = mMouseDownPoint - e->pos();
        if(std::max(std::abs(dis.x()),std::abs(dis.y())) > 40){
            showNormal();
        }
    }
    changeSizeDir(e->x(),e->y());
}

void BaseWindow::addContent(QWidget * w){
    mContainerLay->addWidget(w);
}

int BaseWindow::getTitleBarHeight(){
    return TITILEBAR_HEIGHT;
}

void BaseWindow::mouseDoubleClickEvent(QMouseEvent *event){

    if(QWidget::isMaximized()){
        showNormal();
    }else showMaximized();
}

void BaseWindow::showMaximized(){
    QWidget::showMaximized();
}
void BaseWindow::showNormal(){
    QWidget::showNormal();
}

void BaseWindow::resizeEvent(QResizeEvent *){
    mIsMaximized = isMaximized();
    mRzBtn->refreMode();
}

void BaseWindow::leaveEvent(QEvent *){
}

void BaseWindow::paintEvent(QPaintEvent *){
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    int titleH = TITILEBAR_HEIGHT + RADIUS;
    int w = width();
    int h = height();
    QRect titleRect(1,1,width()-2,titleH);

    painter.save();
    painter.setClipRect(0,0,w,TITILEBAR_HEIGHT);

    QPen pen;
    pen.setWidthF(1.0 / 2.54);
    pen.setColor(mBordColor);
    painter.setPen(pen);
    painter.setBrush(Qt::NoBrush);
    painter.drawRoundedRect(0,0,width()-1,h,RADIUS,RADIUS);//边框

    painter.setBrush(QColor(238,239,245));
    painter.setPen(Qt::NoPen);
    painter.drawRoundedRect(titleRect,RADIUS,RADIUS);

    painter.restore();

    painter.setPen(pen);
    painter.drawLine(0,TITILEBAR_HEIGHT,0,h);
    painter.drawLine(0,h,w,h);
    painter.drawLine(w,TITILEBAR_HEIGHT,w,h);
}

bool BaseWindow::canMove(){
    return !mIsMaximized && !mIsChangeSizeMode;
}

QPoint* BaseWindow::getMovedPoint(){
    mMovedPoint->setX(width() * mDownXPercent);
    return mMovedPoint;
}

void BaseWindow::setWindowEvent(WindowEvent* e){
    this->mWindowEvent = e;
}

void BaseWindow::closeEvent(QCloseEvent *event){
    if(mWindowEvent){
        mWindowEvent->closeEvent();
    }
}

void BaseWindow::changeCursor(){

}
void BaseWindow::changeSizeDir(int x ,int y){
    if(isMaximized()){
        mChangeSizeDir = -1;
        return;
    }
    int w = width();
    int h = height();
    if(x < 0 || x > w || y < 0 || y > h)return;

    if(x < mChangeSizePadding){
        if(y < mChangeSizePadding){
            mChangeSizeDir = 1;// 左上
            setCursor(QCursor(Qt::SizeFDiagCursor));
        }else if(y > h - mChangeSizePadding){
            mChangeSizeDir = 7;//左下
            setCursor(QCursor(Qt::SizeBDiagCursor));
        }else{
            mChangeSizeDir = 0;//左
            setCursor(QCursor(Qt::SizeHorCursor));
        }
    }else if(y < mChangeSizePadding){
        if(x > w - mChangeSizePadding){
            mChangeSizeDir = 3;//右上
            setCursor(QCursor(Qt::SizeBDiagCursor));
        }else{
            mChangeSizeDir = 2;//上
            setCursor(QCursor(Qt::SizeVerCursor));
        }
    }else if(x > w - mChangeSizePadding){
        if(y > h - mChangeSizePadding){//右下
            mChangeSizeDir = 5;
            setCursor(QCursor(Qt::SizeFDiagCursor));
        }else{
            mChangeSizeDir = 4;
            setCursor(QCursor(Qt::SizeHorCursor));
        }
    }else if(y > h - mChangeSizePadding){
        mChangeSizeDir = 6;
        setCursor(QCursor(Qt::SizeVerCursor));
    }else{
        mChangeSizeDir = -1;
        this->setCursor(QCursor(Qt::ArrowCursor));
    }
}

BaseWindow::~BaseWindow(){
    if(mMovedPoint){
        delete mMovedPoint;
    }
}
