#ifndef BASEWINDOW_H
#define BASEWINDOW_H
#include "QWidget"
#include "event/dragevent.h"
#include "QVBoxLayout"
#include "event/WindowEvent.h"
#include "widgets/windowresizebtn.h"

class BaseWindow : public QWidget,public DragEvent
{
private:
    const int RADIUS = 12;
    static const int BUTTON_SIZE = 21;
    static const int TITILEBAR_HEIGHT = 35;
    bool mIsMaximized = false;

    QPoint mMouseDownPoint;
    QPoint mMouseDownWindowGlobalPosition;
    QPoint* mMovedPoint = nullptr;
    QVBoxLayout* mViewLay;
    QVBoxLayout* mContainerLay;
    QHBoxLayout* mTitleLay;
    QHBoxLayout* mTitleContentLay;
    QSize mMaxedSize;
    float mDownXPercent = 0;
    bool mIsChangeSizeMode = false;
    QSize mWhenDownWindowSize;
//    QPoint mWhenDownPoint;
    short mChangeSizeDir = -1;
    const short mChangeSizePadding = 6;
    WindowEvent* mWindowEvent;
    const QColor mBordColor;
    WindowResizeBtn* mRzBtn;
public:
    explicit BaseWindow(QWidget* parent = nullptr);
    void paintEvent(QPaintEvent *event) override;
    void addContent(QWidget*);
    void showMaximized();
    void showNormal();
    static int titleHeight();
    static int buttonSize();
    void mouseMoveEvent(QMouseEvent *) override ;
    void mousePressEvent(QMouseEvent *) override ;
    void mouseReleaseEvent(QMouseEvent *) override ;
    void resizeEvent(QResizeEvent *event) override;
    void leaveEvent(QEvent *) override;
    void changeCursor();
    void changeSizeDir(int x ,int y);
    void mouseDoubleClickEvent(QMouseEvent *event) override;
    bool canMove() override;
    void closeEvent(QCloseEvent *event) override;
    int getTitleBarHeight();
    void setWindowEvent(WindowEvent*);
    QPoint* getMovedPoint() override;
    QHBoxLayout* getTitleLay();
    QHBoxLayout* getTitleContentLay();
    ~BaseWindow();

};

#endif // BASEWINDOW_H
