#include "dragmovedwidget.h"
#include "QMouseEvent"
#include "QtDebug"

DragMovedWidget::DragMovedWidget(QWidget *parent) : QWidget(parent)
{
    setMouseTracking(true);
}

void DragMovedWidget::mousePressEvent(QMouseEvent *e){
    QWidget* host = !mHost ? this : mHost;
    if(e->button() == Qt::LeftButton){
        _canMoveWindow = true;
        mDownLocalPosition = pos();

       mMovedPosition = e->globalPos() - host->pos();
    }

    if(mDragEvent){
        mDragEvent->mousePressEvent(e);
    }
}

void DragMovedWidget::setDragEvent(DragEvent* dragEvent){
    this->mDragEvent = dragEvent;
}

void DragMovedWidget::mouseDoubleClickEvent(QMouseEvent * e){
    if(mDragEvent){
        mDragEvent->mouseDoubleClickEvent(e);
    }
}

void DragMovedWidget::mouseMoveEvent(QMouseEvent *event){
    if(mDragEvent){
        mDragEvent->mouseMoveEvent(event);
    }

    if( _canMoveWindow && (!mDragEvent || (mDragEvent && mDragEvent->canMove()))){
        QWidget* host = !mHost ? this : mHost;
        QPoint* movedPoint = mDragEvent->getMovedPoint();
        host->move(event->globalPos() - (movedPoint ? *movedPoint : mMovedPosition));
    }
    event->accept();
}

void DragMovedWidget::setHost(QWidget * host){
    this->mHost = host;
}

void DragMovedWidget::mouseReleaseEvent(QMouseEvent *event){
    if(mDragEvent){
        mDragEvent->mouseReleaseEvent(event);
    }
    if(event->button() == Qt::LeftButton){
        _canMoveWindow = false;
    }
    event->accept();
}

void DragMovedWidget::leaveEvent(QEvent * event){
    _canMoveWindow = false;
    if(mDragEvent){
        mDragEvent->leaveEvent(event);
    }
}
