#include "expandlistpop.h"
#include "QScrollBar"
#include "QGraphicsDropShadowEffect"
#include "QPainter"
#include "QPushButton"
#include "widgets/adjustlengthpushbutton.h"
#include "QHBoxLayout"
#include "widgets/imageview.h"
#include "dataManager/cappsetting.h"
#include <QRect>

extern CAppSetting* g_cappSetting;
ExpandListPop::ExpandListPop(int width,QWidget * parent):QWidget(parent)
{
    _radiusAble = g_cappSetting->getWindowRadius() > 0;
    if(!_radiusAble){
        mPadding = 0;
        mRadius = 0;
    }
    setFixedWidth(width+mPadding*2);
    mContentWidth = width;

    mList = new QListWidget(this);
    mList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    QScrollBar* scrollBar = new QScrollBar(mList);
    scrollBar->setStyleSheet("QScrollBar:vertical"
    "{"
        "width:4px;"
        "background:rgba(0,0,0,0%);"
        "margin:0px,0px,0px,0px;"
        "padding-top:0px;"   // 留出9px给上面和下面的箭头
        "padding-bottom:0px;"
    "}"
    "QScrollBar::handle:vertical"
    "{"
        "width:4px;"
        "background:rgba(0,0,0,25%);"
        "border-radius:2px;"   // 滚动条两端变成椭圆
        "min-height:20;"
    "}"
     "QScrollBar::add-line:vertical"   // 这个应该是设置下箭头的，3.png就是箭头
     "{"
         "height:9px;width:8px;"
         "border-image:url(:/images/a/3.png);"
         "subcontrol-position:bottom;"
     "}"
     "QScrollBar::sub-line:vertical"   // 设置上箭头
     "{"
         "height:9px;width:8px;"
         "border-image:url(:/images/a/1.png);"
         "subcontrol-position:top;"
     "}"

    );
    mList->setVerticalScrollBar(scrollBar);
//        mList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mList->setFixedWidth(mContentWidth);
    mList->setStyleSheet("QListWidget{border:0px ; background:transparent}"
    "QListWidget::Item:hover{background:#cccccc; }"
    "QListWidget::Item:checked{background:#cccccc; }"

                         );
//        connect(mList,&QListWidget::itemClicked,this,[=](QListWidgetItem * item){
////            if(mItemClick == nullptr)return;

//        });

    QGraphicsDropShadowEffect *shadow_effect = new QGraphicsDropShadowEffect(this);
    shadow_effect->setOffset(0, 0);
    int shadowAlfa = 180;
    #if defined(Q_OS_MACOS)
        shadowAlfa = 100;
    #endif
    shadow_effect->setColor(QColor(0,0,0,shadowAlfa));
    shadow_effect->setBlurRadius(6);

    setGraphicsEffect(shadow_effect);
    setWindowFlags(Qt::Popup);
    setAttribute(Qt::WA_TranslucentBackground);
}

void ExpandListPop::paintEvent(QPaintEvent *) {
    QPainter painter(this);
    painter.setPen(Qt::NoPen);
    painter.setBrush(mBgColor);
    painter.setRenderHint(QPainter::Antialiasing);
//        painter.save();
//        painter.translate(0,-mRadius);
    QRect r = QRect(mPadding,mPadding,width()-mPadding*2,(height() - mPadding) * mFrameAnim-mRadius);
    painter.drawRoundedRect(r,mRadius,mRadius);
//        painter.restore();

    if(!_radiusAble){
        QPen pen;
        pen.setWidth(1);
        pen.setColor(QColor(0xaa,0xaa,0xaa));
        painter.setPen(pen);
        painter.drawRect(rect());
    }
}

void ExpandListPop::setItems(QList<QString> stringItems,QList<ExMenuButtonInfo> menuButtons){
    mLabels.clear();
    mStringItems = stringItems;
    mMenuButtons = menuButtons;
    int labelIndex = 0;
    for(QString text : stringItems){
        AdjustLengthPushButton* label = new AdjustLengthPushButton;
        connect(label,&ClickedLabel::clicked,this,[=]{
            QString s = label->userText();

            int index = 0;
            for(QString&labelText : mStringItems){
                if(labelText == label->userText()){
                    if(mItemClick != nullptr){
                        mItemClick->onItemClick(this,s,labelIndex);
                     }
                    if(mItemClickCallBack){
                        mItemClickCallBack(s,labelIndex);
                    }
                    emit itemClick(s,labelIndex);
                    break;
                }
                index++;
            }
        });

        label->setAttribute(Qt::WA_MacShowFocusRect,false);
        label->setStyleSheet("QLabel{color:#666666;}");

        mLabels.emplace_back(label);
        label->setFixedHeight(mItemHeight);
        label->setText(text);
        QPalette palette;
        palette.setColor(QPalette::Text,QColor(50,50,50));
        label->setPalette(palette);
        label->setPadding(mPaddingLeft,mPaddingRight);
        label->setContentsMargins(mPaddingLeft,mPaddingTop,mPaddingRight,mPaddingBottom);
        QListWidgetItem* item = new QListWidgetItem();
        item->setBackground(Qt::transparent);
        item->setForeground(Qt::transparent);
        item->setSizeHint(QSize(mContentWidth,mItemHeight));
        mList->addItem(item);

        if(menuButtons.size() == stringItems.size()){
            ExMenuButtonInfo exBtnInfo = menuButtons[labelIndex];
            QWidget* itemWidget = new QWidget;
            QHBoxLayout* itemLay = new QHBoxLayout(itemWidget);

            itemLay->setContentsMargins(0,0,mPaddingRight,0);
            itemLay->setSpacing(0);
            itemLay->addWidget(label,1);

            ImageView* image = new ImageView;
            image->waveCenter(true);
            image->setRadius(30);
            image->setFixedSize(30,30);
            image->setPadding(exBtnInfo.paddingLeft,exBtnInfo.paddingTop,exBtnInfo.paddingRight,exBtnInfo.paddingBottom);
            image->setSvgPixMapSrc(exBtnInfo.path,exBtnInfo.width,exBtnInfo.height);
            connect(image,&ImageView::clicked,this,[=]{
                int index = 0;
                for(QString&labelText : mStringItems){
                    if(labelText == label->text()){
                        emit itemMenuClick(index);
                        break;
                    }
                    index++;
                }
            });

            itemLay->addWidget(image);
            mList->setItemWidget(item,itemWidget);
        }else{
            mList->setItemWidget(item,label);
        }

        labelIndex ++;
    }
    int listHeight = min(5,stringItems.size()) *mItemHeight;
    int height = listHeight+mPadding;

    setFixedHeight(height+mPadding+mRadius);
    mList->setFixedHeight(listHeight);
    mList->setGeometry(mPadding,mPadding+mRadius,mList->width(),mList->height());
}

void ExpandListPop::showPop(int x, int y){
//        QTimeLine* timeLine = new QTimeLine(0,this);
//        timeLine->setFrameRange(0,100);
//        timeLine->start();
    mFrameAnim = 1;
//        mList->hide();
//        connect(timeLine,&QTimeLine::frameChanged,this,[=](int f){
//            mFrameAnim = (float)f  / 100;
//            if(f == timeLine->endFrame()){
            mList->show();
//            }
        update();
//        });
    move(x-mPadding,y-mPadding);
    show();
}

void ExpandListPop::removeItem(int index){
    mStringItems.removeAt(index);
    if(!mMenuButtons.isEmpty())mMenuButtons.removeAt(index);
    mList->takeItem(index);
    if(mStringItems.isEmpty()){
        dismiss();
    }else
        setFixedHeight(height() - mItemHeight);
}


