#ifndef EXPANDLISTPOP_H
#define EXPANDLISTPOP_H
#include <QDebug>
#include <QWidget>
#include <QListWidget>
using namespace std;
class ItemClick;
class AdjustLengthPushButton;

class ExMenuButtonInfo{
public:
    QString path;
    bool isSvg;
    int width;
    int height = 0;
    int paddingLeft = 0;
    int paddingTop = 0;
    int paddingRight = 0;
    int paddingBottom = 0;
};

class ExpandListPop : public QWidget
{
    Q_OBJECT
private:
    QList<QString> mStringItems;
    QList<ExMenuButtonInfo> mMenuButtons;
    QListWidget * mList;
    bool _radiusAble = true;
    vector<AdjustLengthPushButton*> mLabels;
    int mItemHeight = 33;
    int mPaddingLeft = 0;
    int mPaddingRight = 0;
    int mPaddingTop = 0;
    int mPaddingBottom = 0;
    bool mIsDismissed = false;
    int mRadius = 6;
    bool mMouseEntered = false;
    int mPadding = 13;
    ItemClick* mItemClick = nullptr;
    std::function<void (const QString&, int index)> mItemClickCallBack = nullptr;
    float mFrameAnim = 0;
    QColor mBgColor = QColor(255,255,255,220);
    int mContentWidth = 0;
public:
    ExpandListPop(int width,QWidget * p=nullptr);
    void removeItem(int index);
    void setItemPadding(int left,int top,int right,int bottom){
        this->mPaddingLeft = left;
        this->mPaddingRight = right;
        this->mPaddingTop = top;
        this->mPaddingBottom = bottom;
    }

    void setItemClickListener(ItemClick* click){
        this->mItemClick = click;
    }

    void setOnItemClickCallback(const std::function<void (const QString&, int index)>& callback){
        mItemClickCallBack = callback;
    }

    void focusInEvent(QFocusEvent *event) override{
        qDebug() << "focusInEvent";
    }

    void focusOutEvent(QFocusEvent *event) override{
        qDebug() << "focusOutEvent";
    }

    void paintEvent(QPaintEvent *) override;

    void setItems(QList<QString> stringItems,QList<ExMenuButtonInfo> menuButtons = {});

    void showPop(int x, int y);

    void dismiss(){
        if(mIsDismissed)return;
        mIsDismissed = true;
        mList->hide();
    //        QTimeLine* timeLine = new QTimeLine(90,this);
    //        timeLine->setFrameRange(100,0);
    //        timeLine->start();
    //        connect(timeLine,&QTimeLine::frameChanged,this,[=](int f){
    //            mFrameAnim = (float)f  / 100;
    //            repaint();
    //            if(f == timeLine->endFrame()){
                this->close();
                deleteLater();
    //            }
    //        });
    }

    void closeEvent(QCloseEvent *) override{
        dismiss();
    }

    void mouseReleaseEvent(QMouseEvent *) override{
        qDebug() << "mouseReleaseEvent";
        if(!mMouseEntered){
            dismiss();
        }
    }

    void leaveEvent(QEvent *) override{
        mMouseEntered = false;
    }

    void enterEvent(QEvent *) override{
        mMouseEntered = true;
    }

    void mouseMoveEvent(QMouseEvent *) override{

    }
    void mousePressEvent(QMouseEvent *) override{
        qDebug() << "mousePressEvent";
    }

    ~ExpandListPop(){
    }

signals:
    void itemClick(QString&, int index);
    void itemMenuClick(int index);
};
class ItemClick{
public:
    virtual void onItemClick(ExpandListPop * pop , QString&, int index) = 0;
};
#endif // EXPANDLISTPOP_H
