#ifndef IMAGEVIEW_H
#define IMAGEVIEW_H
#include "QWidget"
#include "QTimeLine"
#include <iostream>
#include <string>

enum IMVScaleType{
    incenter,
    fillWidth,
    fillHeight,
    fillWH
};
class QMouseEvent;
class ImageView;
typedef void (*ImageVClickListener)(ImageView *,QMouseEvent*);
class ImageClickListener{
public: virtual void onClick(ImageView *,QMouseEvent*) = 0;
};
using namespace std;
class ImageView :public QWidget{
    Q_OBJECT
private:
    IMVScaleType _imv_scaleType = incenter;
    QPixmap _src_map ;
    QPixmap* _map = nullptr;
    QString sign = "";
    bool _reMeasureTextSize = false;
    QSize _textSize;
    QFont _font;
    QColor _fontColor = Qt::black;
    QColor _clickColor = nullptr;
//    QColor _disClickColor = nullptr;
    QColor _drawClickColor = nullptr;
    QColor _waveColor = QColor(0,0,0,15);
    string _trKey = "";
    QString mText = "";
    int _mapWidth = 0;
    int _mapHeight = 0;
    bool isWMax = false;
    int _frame = 0;

    int _downX = 0;
    int _downY = 0;
    bool _waveCenter = false;
    bool _mouseEnter = false;
    QTimeLine* _timerLine = nullptr;
    QTimeLine* _pressTimerLine = nullptr;
    bool _clickable = true;
    bool _isMousePressed = false;
    int _radius = 0;
    qreal _clipRadius = 0;
    int _paddingLeft = 0;
    int _paddingTop = 0;
    int _paddingRight = 0;
    int _paddingBottom = 0;
    bool _sizeInited = false;
    bool _beClicked = false;
    QColor _bgColor;
    ImageVClickListener _clickListener = nullptr;
    ImageClickListener* _onClick = nullptr;
    void * _tag;

public :
    ImageView(QWidget* parent = Q_NULLPTR, Qt::WindowFlags f = Qt::WindowFlags());
    void setTag(void * tag);

    void * getTag();

    void setRadius(int radius);

    void setSign(QString sign);

    QString getSign();

    void paintEvent(QPaintEvent * e) override;


    void drawBackgroudColor(QPainter * painter);

    void setClickListener(ImageVClickListener clickListener);

    void setOnClickListener(ImageClickListener* onClick);

    void setClickable(bool clickable);

    void drawWave();

    void drawImage();

//    void setDisClickableColor(QColor color){
//        this->_disClickColor = color;
//    }

    void setBackgroudColor(QColor backgroudColor);

    void setFontColor(QColor fontColor);

    void setClipRadius(qreal radius);

    void changeEvent(QEvent * e) override;

    void drawText();

    void measureText(QPainter* p);

    void waveCenter(bool center);

    void setLabelFromTrKey(const string& trKey);

    void setLabel(QString text,const string& trKey);

    void setLabelFont(QFont font);

    void changeBgColorByAnim(QColor startColor,QColor endColor);

    bool existBgColr();

    void mousePressEvent(QMouseEvent *e) override;

    void setClickColor(QColor clickColor);

    void mouseReleaseEvent(QMouseEvent * e) override;

    void mouseMoveEvent(QMouseEvent *) override;

    void leaveEvent(QEvent *) override;

    void enterEvent(QEvent *) override;

    void createNewPixmap();

    void freshMapData(double scale);

    void resizeEvent(QResizeEvent *) override;

    void setPixMapSrc(QPixmap map);

    void setSvgPixMapSrc(QString path,int width,int height);

    void setScaleType(IMVScaleType scaleType);

    void setPadding(int left,int top,int right ,int bottom);
    ~ImageView();

    signals:
      void clicked();
      void mouseEvent(int eventCode);//-1 leave 0 enter,1down , 2 move 3 up
};

#endif // IMAGEVIEW_H
