#include "progressdialog.h"
#include "QPainter"
#include "QDateTime"

ProgressDialog::ProgressDialog(QWidget *parent) : QWidget(parent)
{
    if(parent)
        setFixedSize(parent->size());
    else setFixedSize(getDialogSize());
//
    setAttribute(Qt::WA_TranslucentBackground);

}

void ProgressDialog:: showDialog(const QString& string){
    if(_showedMsg){
        setLabel(string);
        return;
    }
    _showedMsg = true;
    _text = string;

    QDateTime dateTime = QDateTime::currentDateTime();
    _showTime = dateTime.time().msec();
    showNormal();
    _lineAnim = new QTimeLine(260,this);
    _lineAnim->setFrameRange(0,100);
    connect(_lineAnim,&QTimeLine::frameChanged,this,[=](int f){
        _bgColor.setAlpha(30.0 * f / _lineAnim->endFrame());
        update();
    });
    _lineAnim->start();

    _progAnim = new QTimeLine(1260,this);

    _progAnim->setFrameRange(-90*16+380/2,16 * 360-90*16+380/2);
    connect(_progAnim,&QTimeLine::frameChanged,this,[=](int f){
        _progress = -f;
        update();
    });
    _progAnim->setLoopCount(0);
    _progAnim->start();
}

void ProgressDialog::setLabel(const QString& text){
    this->_text = text;
    update();
}
void ProgressDialog::setRadius(int radius){
    this->_radius = radius;
}

void ProgressDialog::paintEvent(QPaintEvent *){
     QPainter painter(this);
     painter.setRenderHint(QPainter::Antialiasing);
     painter.setPen(Qt::NoPen);
     int w = width();
     int h = height();
     if(parent()){
          painter.setBrush(_bgColor);
          painter.setRenderHint(QPainter::Antialiasing);
          painter.drawRoundedRect(0,0,w,h,_radius,_radius);
     }

     painter.setBrush(QColor(0,0,0,210));

     QRect rect((w - _dialog_width) / 2,(h - _dialog_height) / 2,_dialog_width,_dialog_height);

     painter.drawRoundedRect(rect,8,8);

     QPen pen;
     pen.setColor(QColor(120,120,120));
     pen.setWidth(2);
     painter.setPen(pen);
     int margin = 33;
     QRect cirRect = QRect(rect.left() + margin,rect.top() + 16,rect.width() - margin * 2,rect.width() - margin * 2);
     painter.drawEllipse(cirRect);
     pen.setColor(QColor(220,220,220));
     painter.setPen(pen);
//     painter.setBrush(Qt::white);
     painter.drawArc(cirRect,_progress,380);

     QFont font;
     font.setPixelSize(12);
    painter.setFont(font);
     int textH = 34;
     painter.drawText(rect.left(),rect.bottom()-textH,rect.width(),textH,Qt::AlignHCenter,_text);
     painter.end();
}

void ProgressDialog::dismiss(){
    _showedMsg = false;
    if(_isDismissed)return;
    close();
    setParent(nullptr);
   _isDismissed = true;
   if(_lineAnim){
       _lineAnim->stop();
    }
   if(_progAnim){
       _progAnim->stop();
   }
}
