#include "qrlabel.h"
#include "QPainter"
#include "QMouseEvent"
#include "utils/widgetutils.h"
#include "QDebug"

QrLabel::QrLabel(QWidget *parent) : QWidget(parent)
{
    mFont.setPixelSize(15);
    mSucPix = QPixmap(":/images/ls.png");
    mSucTxtHeight = WidgetUtils::getTextRect(mFont,mScanSucTxt).height();
}

void QrLabel::paintEvent(QPaintEvent *){
    if(mQrImage.isNull())return;
    QPainter painter(this);

    painter.setFont(mFont);
    int imageWidth = mQrImage.width();
    painter.drawPixmap(mImageX,0,mQrImage);

    if(mIsProgressMode){
        painter.setPen(Qt::NoPen);
        painter.setBrush(QColor(0,0,0,180));
        QRect rect(mImageX,0,imageWidth,imageWidth);
        painter.drawRect(rect);
        painter.setPen(Qt::white);
        painter.setRenderHint(QPainter::Antialiasing);
        if(!mScanSuc)painter.drawText(rect,Qt::AlignCenter,mProgressText);
    }else if(mScanSuc){
        painter.setPen(Qt::NoPen);
        painter.setBrush(QColor(0,0,0,180));
        QRect rect(mImageX,0,imageWidth,imageWidth);
        painter.drawRect(rect);

        int margin = 7;
        int tbPadding = (imageWidth - margin - mSucTxtHeight - mSucPix.height()) / 2;
        int imageLRPadding = (imageWidth - mSucPix.width()) / 2;
        painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        painter.drawPixmap(imageLRPadding+mImageX,tbPadding,mSucPix);

        painter.setPen(Qt::white);
        painter.setRenderHint(QPainter::Antialiasing);
        qDebug() <<mImageX<<tbPadding+mSucPix.height()+margin<<imageWidth<<mSucTxtHeight;
        painter.drawText(mImageX,tbPadding+mSucPix.height()+margin,imageWidth,mSucTxtHeight,Qt::AlignHCenter,mScanSucTxt);
    }
}

void QrLabel::moveImage(int x){
    this->mImageX = x;
    update();
}

int QrLabel::getImageX(){
    return this->mImageX;
}

void QrLabel::setImage(QPixmap &pixmap){
    this->mQrImage = pixmap;
    update();
}

void QrLabel::mouseReleaseEvent(QMouseEvent *event){
    int x = event->x();
    if(x >= mImageX && x <= mImageX){
        emit qrImageClicked();
    }
}

void QrLabel::scanSuccess(){
    mScanSuc = true;
    mIsProgressMode = false;
    update();
}

void QrLabel::setProgress(bool progressMode,const QString& progText){
    this->mProgressText = progText;
    mScanSuc = false;
    this->mIsProgressMode = progressMode;
    update();
}
