#include "tansbyywindow.h"
#include "QTimeLine"
#include "QDebug"

TansByYWindow::TansByYWindow(){
    setWindowFlags(Qt::Window |Qt::FramelessWindowHint);
    setAttribute(Qt::WA_TranslucentBackground);
    setStyleSheet("QGraphicsView{background: transparent;border:0px}");
    mScene = new QGraphicsScene;
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setContentsMargins(0,0,0,0);
    setScene(mScene);
//    setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
}

void TansByYWindow::addContentWidget(QWidget* w){
    mContentView = mScene->addWidget(w);
}

void TansByYWindow::addMenuWidget(QWidget* w){
    mMenuView = mScene->addWidget(w);

}

void TansByYWindow::show(){
    Q_ASSERT(mContentView);
    Q_ASSERT(mMenuView);

    mMenuView->setTransform(QTransform()
                            .translate(mMenuView->boundingRect().width()/2,mMenuView->boundingRect().height()/2)
                            .rotate(-180,Qt::YAxis)
                            .translate(-mMenuView->boundingRect().width()/2,-mMenuView->boundingRect().height()/2));
    mMenuView->setVisible(false);
    QWidget::show();
}

void TansByYWindow::foward(){
    if(!mTimeLine){
        mTimeLine = new QTimeLine(500,this);
        mTimeLine->setStartFrame(0);
        mTimeLine->setEndFrame(180);

        QObject::connect(mTimeLine,&QTimeLine::frameChanged,[=](const int frame){

            mContentView->setTransform(QTransform().translate(mContentView->boundingRect().width()/2,mContentView->boundingRect().height()/2)
                                       .rotate(frame,Qt::YAxis)
                                       .translate(-mContentView->boundingRect().width()/2,-mContentView->boundingRect().height()/2));

            mMenuView->setTransform(QTransform().translate(mMenuView->boundingRect().width()/2,mMenuView->boundingRect().height()/2)
                                       .rotate(frame - 180,Qt::YAxis)
                                       .translate(-mMenuView->boundingRect().width()/2,-mMenuView->boundingRect().height()/2));

            switch (mTimeLine->direction()) {
            case QTimeLine::Forward:
                if(frame > 90 || frame == 90){
                    mContentView->setVisible(false);
                    mMenuView->setVisible(true);
                }
                break;
            case QTimeLine::Backward:
                if(frame < 90 || frame == 90){
                    mContentView->setVisible(true);
                    mMenuView->setVisible(false);
                }
                break;
            }
        });
    }
    mTimeLine->setDirection(QTimeLine::Forward);
    mTimeLine->start();
}

void TansByYWindow::back(){
    if(mTimeLine){
        mTimeLine->setDirection(QTimeLine::Backward);
        mTimeLine->start();
    }
}
