#include "toastwidget.h"
#include "QPainter"
#include "QTimeLine"
#include "QTimer"
#include "QDebug"
#include "utils/widgetutils.h"

ToastWidget::ToastWidget(QWidget* parent):QWidget(parent)
{
    setAttribute(Qt::WA_TransparentForMouseEvents, true);
    if(!parent){
        setWindowFlags(Qt::Window |Qt::FramelessWindowHint);
    }
}

void ToastWidget::showMesssage(const QString& text){
    if(text.isNull() || text.isEmpty())return;
    if(_isShowing){
        _texts.push_back(text);
    }else{
        showMesssagePool(text);
    }
}

void ToastWidget::showMesssagePool(const QString& text){
    if(text.isNull() || text.isEmpty())return;
    this->_text = text;
    _isShowing = true;
    raise();
    this->showNormal();

    QTimeLine* timer = new QTimeLine(300,this);
    timer->setFrameRange(10,255);
    connect(timer,&QTimeLine::frameChanged,this,[=](int frame){
        _bgalpha = frame;
        repaint();
        if(frame == timer->endFrame()){
            QTimer::singleShot(g_duration*1500,this,[=](){
                showMesssagePoolEnd();
            });
        }
    });
    timer->start();
}

void ToastWidget::showMesssagePoolEnd(){
    QTimeLine* timer = new QTimeLine(500,this);
    timer->setFrameRange(255,0);
    connect(timer,&QTimeLine::frameChanged,this,[=](int frame){
        _bgalpha = frame;
        repaint();
//        qDebug() << "showMesssagePoolEnd " << frame;
        if(frame == timer->endFrame()){
            _isShowing = false;
            if(_texts.size() > 0){
                auto first = _texts.begin();
                QString text = *first;
                _texts.erase(first);
                showMesssagePool(text);
            }else {
//                qDebug() << "toast hide";
                if(_autoDelete){
                    deleteLater();
                }else hide();

            }
        }
    });
    timer->start();
}

void ToastWidget::paintEvent(QPaintEvent *){
    QPainter painter(this);
    measureText(_text);

    if(_size.width() == 0)return;

    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(Qt::NoPen);
    painter.setBrush(QColor(0,0,0,_bgalpha));
    painter.drawRoundedRect(rect(),6,6);
    QFont font;
    font.setPixelSize(_fontSize);
    painter.setFont(font);
    painter.setPen(QColor(255,255,255,_bgalpha));
    int line = 0;
    for(QString& s : lines){
        painter.drawText((width() - _size.width()) / 2 ,_txtHeight*0.8+_paddingTB + _txtHeight*line,s);
        line++;
    }
    painter.end();
}

ToastWidget::~ToastWidget(){
}

void ToastWidget::measureTextLines(int width){
    QFont font;
    font.setPixelSize(_fontSize);
    QFontMetrics fontMetrics(font);
    QString targetStr = _measuredText;
    lines.clear();
    int targetWidth = 0;

    while ((targetWidth = fontMetrics.width(targetStr)) > width) {
        QString halfStr = targetStr.mid(0,targetStr.length()/2);
        QString lastHalfStr = targetStr;

        int halfStrWidth = fontMetrics.width(halfStr);
        int lastHalfStrWidth = targetWidth;

        bool widthTooLen = halfStrWidth > width;

        while(widthTooLen){
            lastHalfStr = halfStr;
            lastHalfStrWidth = halfStrWidth;
            halfStr = halfStr.mid(0,halfStr.length()/2);

            halfStrWidth = fontMetrics.width(halfStr);
            widthTooLen = halfStrWidth > width;
        }

        int lalfStrLen = halfStr.length();
        int lastHalfStrLen = lastHalfStr.length();
        int measureLen = width * 1.0 / halfStrWidth * lalfStrLen;
        measureLen = measureLen > lastHalfStrLen ? lastHalfStrLen : measureLen;
        QString measureStr = lastHalfStr.mid(0,measureLen);

        int measureWidth = fontMetrics.width(measureStr);
        if(measureWidth == width){
            lines.push_back(measureStr);
//            qDebug() << "拆解 " << measureStr ;
            targetStr = targetStr.mid(measureLen);
            continue;
        }

        if(measureWidth > width){
            short step = -2;
            measureLen = measureLen + step;
            measureStr = lastHalfStr.mid(0,measureLen);

            measureWidth = fontMetrics.width(measureStr);
            while (measureWidth > width) {
                measureLen = measureLen + step;
                measureStr = lastHalfStr.mid(0,measureLen);

                measureWidth = fontMetrics.width(measureStr);
            }
            QString lastMeasureStr = measureStr;
            measureLen = measureLen + 1;
            measureStr = lastHalfStr.mid(0,measureLen);

            measureWidth = fontMetrics.width(measureStr);
            if(measureWidth > width){
                measureStr = lastMeasureStr;
            }
            lines.push_back(measureStr);

            targetStr = targetStr.mid(measureStr.length());
        }else{
            short step = 2;
            measureLen = measureLen + step;
            measureStr = lastHalfStr.mid(0,measureLen);

            measureWidth = fontMetrics.width(measureStr);
            while (measureWidth < width) {
                measureLen = measureLen + step;
                measureStr = lastHalfStr.mid(0,measureLen);

                measureWidth = fontMetrics.width(measureStr);
            }

            measureLen = measureLen - 1;
            measureStr = lastHalfStr.mid(0,measureLen);

            measureWidth = fontMetrics.width(measureStr);
            if(measureWidth > width){
                measureStr = lastHalfStr.mid(0,measureLen - 1);
            }
//            qDebug() << "拆解 " << measureStr ;
            lines.push_back(measureStr);
            targetStr = targetStr.mid(measureStr.length());
        }
    }
    if(!targetStr.isEmpty())lines.push_back(targetStr);
}

void ToastWidget::measureText(const QString& text){
    if(text.isNull() || text.isEmpty())return;
   if(_measuredText != nullptr && text == _measuredText)return;
    _measuredText = text;
   QFont font;
   font.setPixelSize(_fontSize);
   QRect rect = WidgetUtils::getTextRect(font,text);
   int width = rect.width();//获得一个字符串的宽度

   bool needMeasureMuLine = false;
   if(parentWidget()){
       int pw = parentWidget()->width() * 0.8;
       if(width > pw){
           width = pw;
           needMeasureMuLine = true;
       }
   }

   QFontMetrics fm(font);
   _txtHeight = fm.height();

   if(needMeasureMuLine){
       measureTextLines(width);
   }else {
        lines.clear();
        lines.push_back(_measuredText);
   }

   int height = _txtHeight*lines.size();
   _size = QSize(width,height);
   int w = _size.width()+ _paddingLR * 5;
   int h = _size.height() + _paddingTB * 2;
   setFixedSize(w , h);
   if(parentWidget())setGeometry((parentWidget()->width()-w)/ 2,parentWidget()->height() - h - 30,w,h);
}
