#ifndef WINDOWCLOSEBTN_H
#define WINDOWCLOSEBTN_H
#include <QWidget>
#include <QPainter>
#include <QColor>
#include <cmath>
#include <QApplication>

class WindowCloseButton :public QWidget{
private:
    bool isFocued;
    QWidget* m_host = nullptr;
    void paintEvent(QPaintEvent *) override{
        QPainter painter(this);
        painter.fillRect(rect(),Qt::transparent);
        painter.setRenderHint(QPainter::Antialiasing);

        QColor containColor = QColor(234,94,86,isFocued ? 210 : 250);

        painter.setBrush(containColor);
        painter.setPen(Qt::NoPen);
        int w = width() ;
        int h = height() ;
        painter.drawEllipse(0,0,w,h);

//        int len = sqrt(w * w + h * h);

        QPen pen;
        pen.setWidth(2);
        painter.setPen(pen);

        pen.setColor(Qt::white);

        painter.setPen(pen);
        painter.setBrush(Qt::NoBrush);
        painter.setClipRect(6,6,w - 2*6,w - 2*6);
        painter.save();
//        painter.translate(1,0);
        painter.drawLine(0,0,w,h);
        painter.drawLine(w,0,0,h);

        painter.restore();
    }
public:
    void setHost(QWidget* host){
        this->m_host = host;
    }

    void enterEvent(QEvent *) override{
        isFocued = true;
        repaint();
    }

    void leaveEvent(QEvent *) override{
        isFocued = false;
        repaint();

    }

    void mousePressEvent(QMouseEvent *) override{}

    void mouseMoveEvent(QMouseEvent *) override{}

    void mouseReleaseEvent(QMouseEvent *) override{
        if(isFocued){
            m_host->close();
            #ifndef Q_OS_MAC
                m_host->deleteLater();
            #endif
        }
    }

};
#endif
